@@ -1,13 +1,13 @@
 /* Hey EMACS -*- linux-c -*- */
-/* $Id: images.c 1585 2005-07-28 06:17:03Z roms $ */
+/* $Id: images.c 2417 2007-03-29 01:42:01Z roms $ */
 
-/*  TiEmu - an TI emulator
+/*  TiEmu - Tiemu Is an EMUlator
  *
  *  Copyright (c) 2000-2001, Thomas Corvazier, Romain Lievin
  *  Copyright (c) 2001-2003, Romain Lievin
  *  Copyright (c) 2003, Julien Blache
  *  Copyright (c) 2004, Romain Livin
- *  Copyright (c) 2005, Romain Livin, Kevin Kofler
+ *  Copyright (c) 2005-2007, Romain Livin, Kevin Kofler
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -52,7 +52,7 @@
 #include "ti68k_def.h"
 #include "images.h"
 #include "hwpm.h"
-#include "printl.h"
+#include "logging.h"
 
 #define is_num(c)   isdigit(c)
 #define is_alnum(c) isalnum(c)
@@ -84,10 +84,7 @@ int ti68k_is_a_rom_file(const char *filename)
 
 int ti68k_is_a_tib_file(const char *filename)
 {
-	int ret1 = tifiles_is_a_flash_file(filename);
-	int ret2 = tifiles_is_a_tib_file(filename);
-
-	return ret1 || ret2;
+	return tifiles_file_is_os(filename);
 }
 
 int ti68k_is_a_img_file(const char *filename)
@@ -103,44 +100,57 @@ int ti68k_is_a_img_file(const char *filename)
 	return 0;
 }
 
+int ti68k_is_a_sav_file(const char *filename)
+{
+	char *ext;
+
+	ext = strrchr(filename, '.');
+	if(ext == NULL)
+		return 0;
+	else if(!strcasecmp(ext, ".sav"))
+		return !0;
+
+	return 0;
+}
+
 /*
-	Display informations
+	Display information
 */
 void ti68k_display_rom_infos(IMG_INFO *s)
 {
-	printl(0, _("Rom informations:\n"));
-  	printl(0, _("  Calculator  : %s\n"), ti68k_calctype_to_string(s->calc_type));
-  	printl(0, _("  Firmware    : v%s\n"), s->version);
-  	printl(0, _("  Memory type : %s\n"), ti68k_romtype_to_string(s->flash));
-  	printl(0, _("  Memory size : %iMB (%i bytes)\n"), s->size >> 20, s->size);
-    printl(0, _("  ROM base    : %02x\n"), s->rom_base & 0xff);
-	printl(0, _("  Hardware    : %i\n"), s->hw_type);
+	tiemu_info(_("ROM information:"));
+  	tiemu_info(_("  Calculator  : %s"), ti68k_calctype_to_string(s->calc_type));
+  	tiemu_info(_("  Firmware    : %s"), s->version);
+  	tiemu_info(_("  Memory type : %s"), ti68k_romtype_to_string(s->flash));
+  	tiemu_info(_("  Memory size : %iMB (%i bytes)"), s->size >> 20, s->size);
+    tiemu_info(_("  ROM base    : %02x"), s->rom_base & 0xff);
+	tiemu_info(_("  Hardware    : %i"), s->hw_type);
 }
 
 void ti68k_display_tib_infos(IMG_INFO *s)
 {
-	printl(0, _("Tib informations:\n"));
-  	printl(0, _("  Calculator  : %s\n"), ti68k_calctype_to_string(s->calc_type));
-  	printl(0, _("  Firmware    : v%s\n"), s->version);
-  	printl(0, _("  Memory type : %s\n"), ti68k_romtype_to_string(s->flash));
-  	printl(0, _("  Memory size : %iMB (%i bytes)\n"), s->size >> 20, s->size);
-    printl(0, _("  ROM base    : %02x\n"), s->rom_base & 0xff);
+	tiemu_info(_("TIB information:"));
+  	tiemu_info(_("  Calculator  : %s"), ti68k_calctype_to_string(s->calc_type));
+  	tiemu_info(_("  Firmware    : %s"), s->version);
+  	tiemu_info(_("  Memory type : %s"), ti68k_romtype_to_string(s->flash));
+  	tiemu_info(_("  Memory size : %iMB (%i bytes)"), s->size >> 20, s->size);
+    tiemu_info(_("  ROM base    : %02x"), s->rom_base & 0xff);
 }
 
 void ti68k_display_img_infos(IMG_INFO *s)
 {
-	printl(0, _("Image informations:\n"));
-  	printl(0, _("  Calculator  : %s\n"), ti68k_calctype_to_string(s->calc_type));
-  	printl(0, _("  Firmware    : v%s\n"), s->version);
-  	printl(0, _("  Memory type : %s\n"), ti68k_romtype_to_string(s->flash));
-  	printl(0, _("  Memory size : %iMB (%i bytes)\n"), s->size >> 20, s->size);
-    printl(0, _("  ROM base    : %02x\n"), s->rom_base & 0xff);
-    printl(0, _("  Hardware    : %i\n"), s->hw_type);
-	printl(0, _("  Has boot    : %s\n"), s->has_boot ? "yes" : "no");	
+	tiemu_info(_("Image information:"));
+  	tiemu_info(_("  Calculator  : %s"), ti68k_calctype_to_string(s->calc_type));
+  	tiemu_info(_("  Firmware    : %s"), s->version);
+  	tiemu_info(_("  Memory type : %s"), ti68k_romtype_to_string(s->flash));
+  	tiemu_info(_("  Memory size : %iMB (%i bytes)"), s->size >> 20, s->size);
+    tiemu_info(_("  ROM base    : %02x"), s->rom_base & 0xff);
+    tiemu_info(_("  Hardware    : %i"), s->hw_type);
+	tiemu_info(_("  Has boot    : %s"), s->has_boot ? _("yes") : _("no"));	
 }
 
 /*
-	Get some informations on the ROM dump:
+	Get some information on the ROM dump:
 	- size
 	- ROM base address
 	- FLASH/EPROM
@@ -163,7 +173,7 @@ int ti68k_get_rom_infos(const char *filename, IMG_INFO *rom, int preload)
   	file = fopen(filename, "rb");
   	if(file == NULL)
     {
-      printl(0, _("Unable to open this file: <%s>\n"), filename);
+      tiemu_info(_("Unable to open this file: <%s>"), filename);
       return ERR_CANT_OPEN;
     }
 
@@ -174,6 +184,12 @@ int ti68k_get_rom_infos(const char *filename, IMG_INFO *rom, int preload)
 
   	if(rom->size < 256) 
     	return ERR_INVALID_ROM_SIZE;
+	if (rom->size == 8*MB)
+	{
+	  // TiLP used to dump 8 MB images for HW4, try to load them anyway.
+	  tiemu_info(_("Warning: truncating 8 MB image to 4 MB: <%s>"), filename);
+	  rom->size = 4*MB;
+	}
   	if (rom->size > 4*MB)
     	return ERR_INVALID_ROM_SIZE;
   
@@ -181,9 +197,18 @@ int ti68k_get_rom_infos(const char *filename, IMG_INFO *rom, int preload)
   		rom->data = malloc(rom->size + 4);
 	if(rom->data == NULL)
 		return ERR_MALLOC;
-  	memset(rom->data, 0xff, rom->size);
-  	fread(rom->data, 1, rom->size, file);
-  	fclose(file);
+	memset(rom->data, 0xff, rom->size);
+	if (fread(rom->data, 1, rom->size, file) < (size_t)rom->size)
+	{
+	  tiemu_info(_("Failed to read from file: <%s>"), filename);
+	  fclose(file);
+	  return ERR_CANT_OPEN;
+	}
+	if (fclose(file))
+	{
+	  tiemu_info(_("Failed to close file: <%s>"), filename);
+	  return ERR_CANT_OPEN;
+	}
 
     rom->has_boot = 1;
     rom->rom_base = rom->data[0x05] & 0xf0;
@@ -199,8 +224,9 @@ int ti68k_get_rom_infos(const char *filename, IMG_INFO *rom, int preload)
     else
     {
         // Get hw param block to determine calc type & hw type
-        if(ti68k_get_hw_param_block(rom->data, rom->rom_base, &hwblock) == -1)
-			return ERR_INVALID_ROM;
+        if(ti68k_get_hw_param_block((uint8_t*)rom->data, rom->rom_base, 
+				    &hwblock) == -1)
+	    return ERR_INVALID_ROM;
         ti68k_display_hw_param_block(&hwblock);
 
         switch(hwblock.hardwareID)
@@ -229,7 +255,7 @@ int ti68k_get_rom_infos(const char *filename, IMG_INFO *rom, int preload)
 
 
 /*
-  Get some informations on the FLASH upgrade:
+  Get some information on the FLASH upgrade:
   - size
   - ROM base address
   - os version
@@ -237,8 +263,8 @@ int ti68k_get_rom_infos(const char *filename, IMG_INFO *rom, int preload)
 */
 int ti68k_get_tib_infos(const char *filename, IMG_INFO *tib, int preload)
 {
-	Ti9xFlash content;
-	Ti9xFlash *ptr;
+	FlashContent *content;
+	FlashContent *ptr;
 	int nheaders = 0;
 	int i;
 
@@ -247,22 +273,23 @@ int ti68k_get_tib_infos(const char *filename, IMG_INFO *tib, int preload)
 	   return ERR_CANT_OPEN;
 
 	// Check valid file
-	if(!tifiles_is_a_ti_file(filename))
+	if(!tifiles_file_is_ti(filename))
 		return ERR_NOT_TI_FILE;
 		
-	if(!ti68k_is_a_tib_file(filename))
+	if(!tifiles_file_is_os(filename))
 		return ERR_INVALID_UPGRADE;
 
 	// Load file
-	if(ti9x_read_flash_file(filename, &content) != 0)
+	content = tifiles_content_create_flash(CALC_TI89);
+	if(tifiles_file_read_flash(filename, content) != 0)
         return ERR_INVALID_UPGRADE;
 	
 	// count headers
-  	for (ptr = &content; ptr != NULL; ptr = ptr->next)
+  	for (ptr = content; ptr != NULL; ptr = ptr->next)
     	nheaders++;
   	
   	// keep the last one (data)
-  	for (i = 0, ptr = &content; i < nheaders - 1; i++)
+  	for (i = 0, ptr = content; i < nheaders - 1; i++)
     	ptr = ptr->next;
     	
   	// Load TIB into memory and relocate at SPP
@@ -297,7 +324,7 @@ int ti68k_get_tib_infos(const char *filename, IMG_INFO *tib, int preload)
             }
 		break;
 		default:
-			printl(0, "TIB problem: %02x!\n", 0xff & ptr->device_type);
+			tiemu_info("TIB problem: %02x!\n", 0xff & ptr->device_type);
 			return ERR_INVALID_UPGRADE;
 		break;
 	}
@@ -308,7 +335,7 @@ int ti68k_get_tib_infos(const char *filename, IMG_INFO *tib, int preload)
 
   	get_rom_version(tib->data, tib->size, tib->version);
   	
-  	ti9x_free_flash_content(&content);
+  	tifiles_content_delete_flash(content);
 	if(!preload)
 		free(tib->data);
 
@@ -316,7 +343,7 @@ int ti68k_get_tib_infos(const char *filename, IMG_INFO *tib, int preload)
 }
 
 /*
-	Try to get some informations on the ROM dump:
+	Try to get some information on the ROM dump:
 	- size
 	- ROM base address
 	- FLASH/EPROM
@@ -334,7 +361,7 @@ int ti68k_get_img_infos(const char *filename, IMG_INFO *ri)
 	// Check file
 	if(!ti68k_is_a_img_file(filename))
 	{
-		fprintf(stderr, "Images must have '.img' extension (%s).\n",
+		tiemu_warning("Images must have '.img' extension (%s).\n",
 			filename);
 		return ERR_CANT_OPEN;
 	}
@@ -343,20 +370,30 @@ int ti68k_get_img_infos(const char *filename, IMG_INFO *ri)
   	f = fopen(filename, "rb");
   	if(f == NULL)
     {
-      	fprintf(stderr, "Unable to open this file: <%s>\n", filename);
+      	tiemu_warning("Unable to open this file: <%s>\n", filename);
       	return ERR_CANT_OPEN;
     }
     
     // Read header
-    fread(ri, sizeof(IMG_INFO), 1, f);
-    if(strcmp(ri->signature, "TiEmu img v2.00"))
-   	{
-   		fprintf(stderr, "Bad image: <%s>\n", filename);
-      	return ERR_INVALID_UPGRADE;
-   	}
-    
+    if (fread(ri, sizeof(IMG_INFO), 1, f) < 1)
+    {
+      tiemu_warning("Failed to read from file: <%s>\n", filename);
+      fclose(f);
+      return ERR_CANT_OPEN;
+    }
+
+    if(strcmp(ri->signature, IMG_SIGN) || ri->size > 4*MB)
+    {
+      tiemu_warning("Bad image: <%s>\n", filename);
+      return ERR_INVALID_UPGRADE;
+    }
+
     // Close file
-    fclose(f);
+    if (fclose(f))
+    {
+      tiemu_warning("Failed to close file: <%s>\n", filename);
+      return ERR_CANT_OPEN;
+    }
     
     return 0;
 }
@@ -385,7 +422,7 @@ int ti68k_convert_rom_to_image(const char *srcname, const char *dirname, char **
 	if(err)
     {
 	    free(img.data);
-      	printl(0, _("Unable to get informations on ROM dump: %s\n"), srcname);
+      	tiemu_info(_("Unable to get information on ROM dump: %s"), srcname);
       	return err;
     }
 	ti68k_display_rom_infos(&img);
@@ -403,7 +440,7 @@ int ti68k_convert_rom_to_image(const char *srcname, const char *dirname, char **
   	f = fopen(*dstname, "wb");
   	if(f == NULL)
     {
-      	fprintf(stderr, "Unable to open this file: <%s>\n", *dstname);
+      	tiemu_warning("Unable to open this file: <%s>\n", *dstname);
       	return ERR_CANT_OPEN;
     }
 
@@ -412,23 +449,32 @@ int ti68k_convert_rom_to_image(const char *srcname, const char *dirname, char **
 	{
 		img.size = 4*MB;
 		img.data = realloc(img.data, 4*MB + 4);
-		printf("Completing image to 4MB !\n");
+		tiemu_info(_("Completing image to 4 MB!"));
 		memset(img.data + 2*MB, 0xff, 2*MB);
 	}
 
 	// Fill header
-	strcpy(img.signature, "TiEmu img v2.00");
+	strcpy(img.signature, IMG_SIGN);
 	img.header_size = sizeof(IMG_INFO);
     img.revision = IMG_REV;
 
 	// Write file
-	fwrite(&img, 1, sizeof(IMG_INFO), f);
-	fwrite(img.data, sizeof(char), img.size, f);
+	if (fwrite(&img, 1, sizeof(IMG_INFO), f) < (size_t)sizeof(IMG_INFO)
+	    || fwrite(img.data, sizeof(char), img.size, f) < (size_t)img.size)
+	{
+	  tiemu_warning("Failed to write to file: <%s>\n", *dstname);
+	  fclose(f);
+	  return ERR_CANT_OPEN;
+	}
 
 	// Close file
-	fclose(f);
+	if (fclose(f))
+	{
+	  tiemu_warning("Failed to close file: <%s>\n", *dstname);
+	  return ERR_CANT_OPEN;
+	}
 
-  	return 0;
+	return 0;
 }
 
 /*
@@ -460,7 +506,7 @@ int ti68k_convert_tib_to_image(const char *srcname, const char *dirname, char **
 	if(err)
     {
 	    free(img.data);
-      	printl(0, _("Unable to get informations on FLASH upgrade: %s\n"), srcname);
+      	tiemu_info(_("Unable to get information on FLASH upgrade: <%s>"), srcname);
       	return err;
     }
 	ti68k_display_tib_infos(&img);
@@ -478,29 +524,42 @@ int ti68k_convert_tib_to_image(const char *srcname, const char *dirname, char **
   	f = fopen(*dstname, "wb");
   	if(f == NULL)
     {
-      	fprintf(stderr, "Unable to open this file: <%s>\n", *dstname);
+      	tiemu_warning("Unable to open this file: <%s>\n", *dstname);
       	return ERR_CANT_OPEN;
     }
 
 	// Fill header
-	strcpy(img.signature, "TiEmu img v2.00");
+	strcpy(img.signature, IMG_SIGN);
 	img.header_size = sizeof(IMG_INFO);
 	img.revision = IMG_REV;
     real_size = img.size - SPP;
     img.size = ti68k_get_rom_size(img.calc_type);
-    if(img.calc_type == TI89t)
-        img.hw_type = HW3;  //default
-    else if(hw_type == -1)
-        img.hw_type = HW2;  //default
-	else
-		img.hw_type = hw_type;
+
+    img.hw_type = hw_type;
+	if(hw_type == -1)
+	{
+		if(img.calc_type == TI89t)
+			img.hw_type = HW3;  //default
+		else if(img.calc_type == TI89 || img.calc_type == TI92p || img.calc_type == V200)
+			img.hw_type = HW2;	// default
+	}
 	
 	// Write header
-	fwrite(&img, 1, sizeof(IMG_INFO), f);
-  
-  	// Write boot block
-    memcpy(img.data, &img.data[SPP + BO], 256);
-    fwrite(img.data, 1, 256, f);
+	if (fwrite(&img, 1, sizeof(IMG_INFO), f) < sizeof(IMG_INFO))
+	{
+	  tiemu_warning("Failed to write to file: <%s>\n", *dstname);
+	  fclose(f);
+	  return ERR_CANT_OPEN;
+	}
+ 
+	// Write boot block
+	memcpy(img.data, &img.data[SPP + BO], 256);
+	if (fwrite(img.data, 1, 256, f) < 256)
+	{
+	  tiemu_warning("Failed to write to file: <%s>\n", *dstname);
+	  fclose(f);
+	  return ERR_CANT_OPEN;
+	}
 
     // Write hardware param block
 
@@ -527,72 +586,101 @@ int ti68k_convert_tib_to_image(const char *srcname, const char *dirname, char **
 	}
 	hwpb.bootMajor = hwpb.bootRevision = hwpb.bootBuild = 1;
 	hwpb.gateArray = img.hw_type;
-	ti68k_put_hw_param_block(img.data, img.rom_base, &hwpb);
+	ti68k_put_hw_param_block((uint8_t *)img.data, img.rom_base, &hwpb);
 
 	// write filler
-	fputc(0xfe, f); fputc(0xed, f); fputc(0xba, f); fputc(0xbe, f);
+	if (fputc(0xfe, f) < 0 || fputc(0xed, f) < 0 || fputc(0xba, f) < 0 || fputc(0xbe, f) < 0
 	//fwrite(&hwpb, 1hwpb.len+2, f);
 
 	// write address (pointer)
-    fputc(0x00, f);
-    fputc(img.rom_base, f);
-    fputc(0x01, f);
-    fputc(0x08, f);
+	|| fputc(0x00, f) < 0
+	|| fputc(img.rom_base, f) < 0
+	|| fputc(0x01, f) < 0
+	|| fputc(0x08, f) < 0
 
 	// write structure
-	fputc(MSB(hwpb.len), f);	
-	fputc(LSB(hwpb.len), f);
-	fputc(MSB(MSW(hwpb.hardwareID)), f);
-	fputc(LSB(MSW(hwpb.hardwareID)), f);
-	fputc(MSB(LSW(hwpb.hardwareID)), f);
-	fputc(LSB(LSW(hwpb.hardwareID)), f);
-	fputc(MSB(MSW(hwpb.hardwareRevision)), f);
-	fputc(LSB(MSW(hwpb.hardwareRevision)), f);
-	fputc(MSB(LSW(hwpb.hardwareRevision)), f);
-	fputc(LSB(LSW(hwpb.hardwareRevision)), f);
-	fputc(MSB(MSW(hwpb.bootMajor)), f);
-	fputc(LSB(MSW(hwpb.bootMajor)), f);
-	fputc(MSB(LSW(hwpb.bootMajor)), f);
-	fputc(LSB(LSW(hwpb.bootMajor)), f);
-	fputc(MSB(MSW(hwpb.hardwareRevision)), f);
-	fputc(LSB(MSW(hwpb.hardwareRevision)), f);
-	fputc(MSB(LSW(hwpb.hardwareRevision)), f);
-	fputc(LSB(LSW(hwpb.hardwareRevision)), f);
-	fputc(MSB(MSW(hwpb.bootBuild)), f);
-	fputc(LSB(MSW(hwpb.bootBuild)), f);
-	fputc(MSB(LSW(hwpb.bootBuild)), f);
-	fputc(LSB(LSW(hwpb.bootBuild)), f);
-	fputc(MSB(MSW(hwpb.gateArray)), f);
-	fputc(LSB(MSW(hwpb.gateArray)), f);
-	fputc(MSB(LSW(hwpb.gateArray)), f);
-	fputc(LSB(LSW(hwpb.gateArray)), f);
+	|| fputc(MSB(hwpb.len), f) < 0
+	|| fputc(LSB(hwpb.len), f) < 0
+	|| fputc(MSB(MSW(hwpb.hardwareID)), f) < 0
+	|| fputc(LSB(MSW(hwpb.hardwareID)), f) < 0
+	|| fputc(MSB(LSW(hwpb.hardwareID)), f) < 0
+	|| fputc(LSB(LSW(hwpb.hardwareID)), f) < 0
+	|| fputc(MSB(MSW(hwpb.hardwareRevision)), f) < 0
+	|| fputc(LSB(MSW(hwpb.hardwareRevision)), f) < 0
+	|| fputc(MSB(LSW(hwpb.hardwareRevision)), f) < 0
+	|| fputc(LSB(LSW(hwpb.hardwareRevision)), f) < 0
+	|| fputc(MSB(MSW(hwpb.bootMajor)), f) < 0
+	|| fputc(LSB(MSW(hwpb.bootMajor)), f) < 0
+	|| fputc(MSB(LSW(hwpb.bootMajor)), f) < 0
+	|| fputc(LSB(LSW(hwpb.bootMajor)), f) < 0
+	|| fputc(MSB(MSW(hwpb.hardwareRevision)), f) < 0
+	|| fputc(LSB(MSW(hwpb.hardwareRevision)), f) < 0
+	|| fputc(MSB(LSW(hwpb.hardwareRevision)), f) < 0
+	|| fputc(LSB(LSW(hwpb.hardwareRevision)), f) < 0
+	|| fputc(MSB(MSW(hwpb.bootBuild)), f) < 0
+	|| fputc(LSB(MSW(hwpb.bootBuild)), f) < 0
+	|| fputc(MSB(LSW(hwpb.bootBuild)), f) < 0
+	|| fputc(LSB(LSW(hwpb.bootBuild)), f) < 0
+	|| fputc(MSB(MSW(hwpb.gateArray)), f) < 0
+	|| fputc(LSB(MSW(hwpb.gateArray)), f) < 0
+	|| fputc(MSB(LSW(hwpb.gateArray)), f) < 0
+	|| fputc(LSB(LSW(hwpb.gateArray)), f) < 0)
+	{
+	  tiemu_warning("Failed to write to file: <%s>\n", *dstname);
+	  fclose(f);
+	  return ERR_CANT_OPEN;
+	}
 
 	// Fill with 0xff up-to System Part
-  	for(i = 0x108 + hwpb.len+2; i < SPP; i++)
-    	fputc(0xff, f);
-  
-  	// Copy FLASH upgrade at 0x12000 (SPP)
-  	num_blocks = real_size / 65536;
-  	for(i = 0; i < num_blocks; i++ )
-    {
-      	printl(0, ".");
-      	fflush(stdout);
+	for(i = 0x108 + hwpb.len+2; i < SPP; i++)
+		if (fputc(0xff, f) < 0)
+		{
+		  tiemu_warning("Failed to write to file: <%s>\n", *dstname);
+		  fclose(f);
+		  return ERR_CANT_OPEN;
+		}
+ 
+	// Copy FLASH upgrade at 0x12000 (SPP)
+	num_blocks = real_size / 65536;
+	for(i = 0; i < num_blocks; i++ )
+	{
+		tiemu_info(".");
+		fflush(stdout);
 
-      	fwrite(&img.data[65536 * i + SPP], sizeof(char), 65536, f);
-    }
+		if (fwrite(&img.data[65536 * i + SPP], sizeof(char), 65536, f) < 65536)
+		{
+		  tiemu_warning("Failed to write to file: <%s>\n", *dstname);
+		  fclose(f);
+		  return ERR_CANT_OPEN;
+		}
+	}
 
-  	last_block = real_size % 65536;
-   	fwrite(&img.data[65536 * i + SPP], sizeof(char), last_block, f);
-  
-  	printl(0, "\n");
-  	printl(0, "Completing to %iMB size\n", img.size >> 20);
-  	for(j = SPP + real_size; j < img.size; j++)
-  		fputc(0xff, f);
-  
-  	// Close file
-  	fclose(f);
+	last_block = real_size % 65536;
+	if (fwrite(&img.data[65536 * i + SPP], sizeof(char), last_block, f) < (size_t)last_block)
+	{
+	  tiemu_warning("Failed to write to file: <%s>\n", *dstname);
+	  fclose(f);
+	  return ERR_CANT_OPEN;
+	}
+ 
+	tiemu_info("");
+	tiemu_info("Completing to %iMB size\n", img.size >> 20);
+	for(j = SPP + real_size; j < img.size; j++)
+		if (fputc(0xff, f) < 0)
+		{
+		  tiemu_warning("Failed to write to file: <%s>\n", *dstname);
+		  fclose(f);
+		  return ERR_CANT_OPEN;
+		}
+ 
+	// Close file
+	if (fclose(f))
+	{
+	  tiemu_warning("Failed to close file: <%s>\n", *dstname);
+	  return ERR_CANT_OPEN;
+	}
 
-  	return 0;
+	return 0;
 }
 
 /*
@@ -624,7 +712,7 @@ int ti68k_merge_rom_and_tib_to_image(const char *srcname1, const char *srcname2,
 	if(err)
     {
 	    free(img.data);
-      	printl(0, _("Unable to get informations on ROM dump: %s\n"), srcname1);
+      	tiemu_info(_("Unable to get information on ROM dump: %s"), srcname1);
       	return err;
     }
 	ti68k_display_rom_infos(&img);
@@ -638,7 +726,7 @@ int ti68k_merge_rom_and_tib_to_image(const char *srcname1, const char *srcname2,
 	if(err)
     {
 	    free(img.data);
-      	printl(0, _("Unable to get informations on ROM dump: %s\n"), srcname2);
+      	tiemu_info(_("Unable to get information on ROM dump: %s"), srcname2);
       	return err;
     }
 	ti68k_display_tib_infos(&img);
@@ -656,27 +744,36 @@ int ti68k_merge_rom_and_tib_to_image(const char *srcname1, const char *srcname2,
     img.size = real_size;
 
 	// Open dest file
-  	f = fopen(*dstname, "wb");
-  	if(f == NULL)
-    {
-      	fprintf(stderr, "Unable to open this file: <%s>\n", *dstname);
-      	return ERR_CANT_OPEN;
-    }
+	f = fopen(*dstname, "wb");
+	if(f == NULL)
+	{
+		tiemu_warning("Unable to open this file: <%s>\n", *dstname);
+		return ERR_CANT_OPEN;
+	}
 
 	// Fill header
-	strcpy(img.signature, "TiEmu img v2.00");
+	strcpy(img.signature, IMG_SIGN);
 	img.header_size = sizeof(IMG_INFO);
 	img.revision = IMG_REV;
-    img.has_boot = 1;
+	img.has_boot = 1;
 
 	// Write file
-	fwrite(&img, 1, sizeof(IMG_INFO), f);
-	fwrite(img.data, sizeof(char), img.size, f);
+	if (fwrite(&img, 1, sizeof(IMG_INFO), f) < sizeof(IMG_INFO)
+	    || fwrite(img.data, sizeof(char), img.size, f) < (size_t)img.size)
+	{
+	  tiemu_warning("Failed to write to file: <%s>\n", *dstname);
+	  fclose(f);
+	  return ERR_CANT_OPEN;
+	}
 
 	// Close file
-	fclose(f);
+	if (fclose(f))
+	{
+	  tiemu_warning("Failed to close file: <%s>\n", *dstname);
+	  return ERR_CANT_OPEN;
+	}
 
-  	return 0;
+	return 0;
 }
 
 
@@ -700,7 +797,7 @@ int ti68k_load_image(const char *filename)
 	err = ti68k_get_img_infos(filename, img);
   	if(err)
     {
-      	printl(0, _("Unable to get informations on image: %s\n"), filename);
+      	tiemu_info(_("Unable to get information on image: %s"), filename);
       	return err;
     }
 	ti68k_display_img_infos(img);
@@ -708,31 +805,49 @@ int ti68k_load_image(const char *filename)
 	// Open file
 	f = fopen(filename, "rb");
 	if(f == NULL)
-    {
-      	fprintf(stderr, "Unable to open this file: <%s>\n", filename);
-      	return ERR_CANT_OPEN;
-    }
+	{
+		tiemu_warning("Unable to open this file: <%s>\n", filename);
+		return ERR_CANT_OPEN;
+	}
 
 	// Read pure data
-    fseek(f, img->header_size, SEEK_SET);
+	if (fseek(f, img->header_size, SEEK_SET))
+	{
+		tiemu_warning("Failed to read from file: <%s>\n", filename);
+		fclose(f);
+		return ERR_CANT_OPEN;
+	}
+
 	img->data = malloc(img->size + 4);
 	if(img->data == NULL)
 		return ERR_MALLOC;
-    fread(img->data, 1, img->size, f);
+	if (fread(img->data, 1, img->size, f) < (size_t)img->size)
+	{
+		tiemu_warning("Failed to read from file: <%s>\n", filename);
+		fclose(f);
+		return ERR_CANT_OPEN;
+	}
 
 #if 1
 	{
 		HW_PARM_BLOCK hwblock;
 
-		ti68k_get_hw_param_block(img->data, img->rom_base, &hwblock);
-        ti68k_display_hw_param_block(&hwblock);
+		ti68k_get_hw_param_block((uint8_t *)img->data, img->rom_base, 
+					 &hwblock);
+		ti68k_display_hw_param_block(&hwblock);
 	}
 #endif
- 
-  	img_loaded = 1;
+
+	if (fclose(f))
+	{
+		tiemu_warning("Failed to close file: <%s>\n", filename);
+		return ERR_CANT_OPEN;
+	}
+
+	img_loaded = 1;
 	img_changed = 1;
 
-  	return 0;
+	return 0;
 }
 
 /*
@@ -758,7 +873,7 @@ int ti68k_load_upgrade(const char *filename)
 	if(err)
     {
 		free(img->data);
-      	printl(0, _("Unable to get informations on FLASH upgrade: %s\n"), filename);
+      	tiemu_info(_("Unable to get information on FLASH upgrade: <%s>"), filename);
       	return err;
     }
 	ti68k_display_tib_infos(&tib);
@@ -817,7 +932,7 @@ int ti68k_scan_files(const char *src_dir, const char *dst_dir, int erase)
 	dir = g_dir_open(src_dir, 0, &error);
 	if (dir == NULL) 
 	{
-		fprintf(stderr, _("Opendir error\n"));
+		tiemu_warning(_("Opendir error"));
       	return ERR_CANT_OPEN_DIR;
 	}
 
@@ -883,13 +998,13 @@ int ti68k_scan_images(const char *dirname, const char *filename)
 	struct stat f_info;
   	char *line[7];
 
-  	printl(0, _("Scanning images/upgrades... "));
+  	tiemu_info(_("Scanning images/upgrades... "));
 
 	// Create file (and overwrite)
 	file = fopen(filename, "wt");
     if(file == NULL)
 	{
-	  	fprintf(stderr, _("Unable to open this file: <%s>\n"), filename);
+	  	tiemu_warning(_("Unable to open this file: <%s>"), filename);
 	  	return ERR_CANT_OPEN;
 	} 	
 
@@ -897,7 +1012,7 @@ int ti68k_scan_images(const char *dirname, const char *filename)
 	dir = g_dir_open(dirname, 0, &error);
 	if (dir == NULL) 
 	{
-		fprintf(stderr, _("Opendir error\n"));
+		tiemu_warning(_("Opendir error"));
       	return ERR_CANT_OPEN_DIR;
 	}
   
@@ -911,7 +1026,7 @@ int ti68k_scan_images(const char *dirname, const char *filename)
 		ret = stat(path, &f_info);
 		if(ret == -1)
 		{
-			fprintf(stderr, _("Can not stat: <%s>\n"), dirent);
+			tiemu_warning(_("Can not stat: <%s>"), dirent);
 	      	perror("stat: ");
 		}
 		else
@@ -922,7 +1037,7 @@ int ti68k_scan_images(const char *dirname, const char *filename)
 				ret = ti68k_get_img_infos(path, &img);
 				if(ret)
 				{
-					fprintf(stderr, _("Can not get ROM/update info: <%s>\n"), path);
+					tiemu_warning(_("Can not get ROM/update info: <%s>"), path);
 					break;
 				}
 			}
@@ -951,7 +1066,7 @@ int ti68k_scan_images(const char *dirname, const char *filename)
 	g_dir_close(dir);
   
   	fclose(file);
-  	printl(0, _("Done.\n"));
+  	tiemu_info(_("Done."));
   
   	return 0;
 }
@@ -1035,7 +1150,7 @@ int ti68k_find_image(const char *dirname, char **dst_name)
 	dir = g_dir_open(dirname, 0, &error);
 	if (dir == NULL) 
 	{
-		fprintf(stderr, _("Opendir error\n"));
+		tiemu_warning(_("Opendir error"));
       	return ERR_CANT_OPEN_DIR;
 	}
 
