@@ -1,13 +1,13 @@
 /* Hey EMACS -*- linux-c -*- */
-/* $Id: interface.c 1653 2005-08-07 08:02:17Z roms $ */
+/* $Id: interface.c 2571 2007-06-25 20:34:10Z roms $ */
 
-/*  TiEmu - an TI emulator
+/*  TiEmu - Tiemu Is an EMUlator
  *
  *  Copyright (c) 2000-2001, Thomas Corvazier, Romain Lievin
  *  Copyright (c) 2001-2003, Romain Lievin
  *  Copyright (c) 2003, Julien Blache
  *  Copyright (c) 2004, Romain Livin
- *  Copyright (c) 2005, Romain Livin, Kevin Kofler
+ *  Copyright (c) 2005-2007, Romain Livin, Kevin Kofler
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -45,13 +45,17 @@
 #include "hw.h"
 #include "m68k.h"
 #include "dbus.h"
-#include "printl.h"
+#include "logging.h"
+#ifndef NO_SOUND
+#include "audio.h"
+#endif
 
 #include "ti68k_int.h"
 #include "ti68k_err.h"
 #include "mem_size.h"
 #include "romcalls.h"
 #include "iodefs.h"
+#include "mem_map.h"
 
 /**********************/
 /* Internal variables */
@@ -59,10 +63,10 @@
 
 
 Ti68kParameters     params = { 0 };
-Ti68kHardware       tihw = { 0 };
-TicableLinkParam    link_cable = { 0 };
-Ti68kBreakpoints	bkpts = { 0 };
-
+Ti68kHardware       tihw   = { 0 };
+Ti68kLinkPort	    linkp  = { 0 };
+Ti68kBreakpoints	bkpts  = { 0 };
+Ti68kLogging		logger = { 0 };
 
 /***********************************/
 /* Entry points for initialization */
@@ -98,18 +102,21 @@ static int is_win_9x(void)
  */
 int ti68k_config_load_default(void)
 {
-    params.restricted = 1;
-    params.cpu_rate = -1;
-    params.hw_rate = -1;
-    params.lcd_rate = -1;
-	params.hw_protect = !0;
+	params.restricted = 1;
+	params.cpu_rate = -1;
+	params.hw_rate = -1;
+	params.lcd_rate = -1;
+	params.hw_protect = 1;
 	params.recv_file = 1;
+#ifndef NO_SOUND
+	audio_disable();
+#endif
 
-	params.timeout = is_win_9x() ? 600 : 15;	// 1.5 or 60s
-
-    ticable_get_default_param(&link_cable);
-    link_cable.link_type = LINK_NUL;
-    link_cable.port = NULL_PORT;
+	linkp.cable_delay = DFLT_DELAY;
+	linkp.cable_timeout = is_win_9x() ? 600 : DFLT_TIMEOUT;
+	linkp.cable_port = PORT_1;
+	linkp.cable_model = CABLE_ILP;
+	linkp.calc_model = ti68k_calc_to_libti_calc();
 
     return 0;
 }
@@ -127,9 +134,9 @@ int ti68k_load_image(const char *filename);
 int ti68k_init(void)
 {
 	// init libs
-    ticable_init();
-	tifiles_init();
-	ticalc_init();
+    ticables_library_init();
+	tifiles_library_init();
+	ticalcs_library_init();
 
 	// check if image has been loaded
 	if(img_loaded == 0)
@@ -162,9 +169,9 @@ int ti68k_exit(void)
 {
     TRY(hw_exit());
 
-    ticable_exit();
-	tifiles_exit();
-	ticalc_exit();
+    ticables_library_exit();
+	tifiles_library_exit();
+	ticalcs_library_exit();
 
 	return 0;
 }
@@ -192,12 +199,29 @@ int ti68k_linkport_send_file(const char *filename)
     return send_ti_file(filename);
 }
 
+int ti68k_linkport_unconfigure(void)
+{
+	return hw_dbus_exit();
+}
+
 int ti68k_linkport_reconfigure(void)
 {
-	TRY(hw_dbus_exit());
-    TRY(hw_dbus_init());
+	return hw_dbus_init();
+}
 
-    return 0;
+int ti68k_calc_to_libti_calc()
+{
+	switch(tihw.calc_type)
+	{
+    case TI89:  return CALC_TI89;  break;
+	case TI89t: return CALC_TI89T; break;
+	case TI92:  return CALC_TI92;  break;
+	case TI92p: return CALC_TI92P; break;
+	case V200:  return CALC_V200;  break;
+	default: return CALC_NONE; break;
+	}
+
+	return CALC_NONE;
 }
 
 /******************/
@@ -225,7 +249,7 @@ int ti68k_get_rom_size(int calc_type)
 {
     if(calc_type > CALC_MAX)
     {
-        printl(0, "Bad argument !\n");
+        tiemu_error(_("Bad argument!"));
         exit(0);
     }
 
@@ -236,7 +260,7 @@ int ti68k_get_ram_size(int calc_type)
 {
     if(calc_type > CALC_MAX)
     {
-        printl(0, "Bad argument !\n");
+        tiemu_error(_("Bad argument!"));
         exit(0);
     }
 
@@ -247,7 +271,7 @@ int ti68k_get_io_size(int calc_type)
 {
 	if(calc_type > CALC_MAX)
     {
-        printl(0, "Bad argument !\n");
+        tiemu_error(_("Bad argument!"));
         exit(0);
     }
 
@@ -258,7 +282,7 @@ int ti68k_get_io2_size(int calc_type)
 {
 	if(calc_type > CALC_MAX)
     {
-        printl(0, "Bad argument !\n");
+        tiemu_error(_("Bad argument!"));
         exit(0);
     }
 
@@ -269,7 +293,7 @@ int ti68k_get_io3_size(int calc_type)
 {
 	if(calc_type > CALC_MAX)
     {
-        printl(0, "Bad argument !\n");
+        tiemu_error(_("Bad argument!"));
         exit(0);
     }
 
@@ -280,6 +304,14 @@ int ti68k_get_io3_size(int calc_type)
 /* Misc */
 /********/
 
+void ti68k_unprotect_64KB_range(uint32_t addr)
+{
+    unsigned blockid = addr >> 12, i;
+
+    for (i = blockid; i <= blockid + 16 && i < 64; i++)
+        tihw.ram_exec[i] = 0;
+}
+
 int ti68k_debug_load_symbols(const char *path)
 {
 	if(!strcmp(path, ""))
@@ -295,3 +327,11 @@ int ti68k_debug_load_iodefs(const char *path)
 
 	return iodefs_load(path);
 }
+
+int ti68k_debug_load_memmap(const char *path)
+{
+	if(!strcmp(path, ""))
+		return 0;
+
+	return memmap_load(path);
+}
\ No newline at end of file
