@@ -1,5 +1,5 @@
 /* Hey EMACS -*- linux-c -*- */
-/* $Id: newcpu.c 1547 2005-06-25 16:15:46Z kevinkofler $ */
+/* $Id: newcpu.c 2679 2007-11-18 08:31:34Z roms $ */
 
  /*
   * UAE - The Un*x Amiga Emulator
@@ -23,14 +23,13 @@
 // tiemu begin
 #include "tilibs.h"
 #include "ti68k_def.h"
-#include "memory.h"
-#include "../ti_hw/dbus.h"
 #include "bkpts.h"
 #define write_log printf
 static const struct uae_prefs currprefs = {0, 1, 1};
-#if CYGNUS_SIM
+#if defined(CYGNUS_SIM) && !defined(NO_GDB)
 extern const char *symfile;
 #endif /* CYGNUS_SIM */
+#define FLOATFORMAT_H /* don't include glib.h in romcalls.h */
 // tiemu end
 
 /* Opcode of faulting instruction */
@@ -129,11 +128,10 @@ static void build_cpufunctbl (void)
 			  : currprefs.cpu_level == 3 ? op_smalltbl_1_ff
 			  : currprefs.cpu_level == 2 ? op_smalltbl_2_ff
 			  : currprefs.cpu_level == 1 ? op_smalltbl_3_ff
-			  : ! currprefs.cpu_compatible ? op_smalltbl_4_ff
 			  : op_smalltbl_5_ff);
 
-    write_log ("UAE: Building CPU function table (%d %d %d).\n",
-	       currprefs.cpu_level, currprefs.cpu_compatible, currprefs.address_space_24);
+    write_log ("UAE: Building CPU function table (%d %d).\n",
+	       currprefs.cpu_level, currprefs.address_space_24);
 
     for (opcode = 0; opcode < 65536; opcode++)
 	cpufunctbl[opcode] = op_illg_1;
@@ -190,16 +188,6 @@ static void update_68k_cycles (void)
 
 void check_prefs_changed_cpu (void)
 {
-    if (currprefs.cpu_level != changed_prefs.cpu_level
-	|| currprefs.cpu_compatible != changed_prefs.cpu_compatible)
-    {
-	if (!currprefs.cpu_compatible && changed_prefs.cpu_compatible)
-	    fill_prefetch_slow ();
-
-	currprefs.cpu_level = changed_prefs.cpu_level;
-	currprefs.cpu_compatible = changed_prefs.cpu_compatible;
-	build_cpufunctbl ();
-    }
     if (currprefs.m68k_speed != changed_prefs.m68k_speed) {
 	currprefs.m68k_speed = changed_prefs.m68k_speed;
 	reset_frame_rate_hack ();
@@ -212,7 +200,7 @@ void init_m68k (void)
 {
     int i;
 
-    printf ("UAE: version 0.8.23\n");
+    printf ("UAE: version 0.8.25\n");
 //    update_68k_cycles ();
 
     for (i = 0 ; i < 256 ; i++) {
@@ -271,8 +259,6 @@ void init_m68k (void)
         write_log ("000");
         break;
     }
-    if (currprefs.cpu_compatible)
-        write_log (" (compatible mode)");
     if (currprefs.address_space_24) {
 	regs.address_space_mask = 0x00ffffff;
 	write_log (" 24-bit addressing");
@@ -288,16 +274,38 @@ void init_m68k (void)
 }
 
 struct regstruct regs, lastint_regs;
+#ifndef NO_GDB
 static struct regstruct regs_backup[16];
 static int backup_pointer = 0;
+#endif /* 0 */
+#ifdef NO_GDB
 static long int m68kpc_offset;
+#endif /* NO_GDB */
 int lastint_no;
 
 #define get_ibyte_1(o) get_byte(regs.pc + (regs.pc_p - regs.pc_oldp) + (o) + 1)
 #define get_iword_1(o) get_word(regs.pc + (regs.pc_p - regs.pc_oldp) + (o))
 #define get_ilong_1(o) get_long(regs.pc + (regs.pc_p - regs.pc_oldp) + (o))
 
-#if 0
+#ifdef NO_GDB
+
+#ifdef __WIN32__
+#define snprintf	_snprintf
+#endif
+
+char *sym_addr(uae_u32 addr)
+{
+	static char buf[256];
+	int rcid = romcalls_is_addr(addr);
+
+	if(rcid == -1)
+		snprintf(buf, sizeof(buf), "$%06lX", (unsigned long)addr);
+	else
+		snprintf(buf, sizeof(buf), "$%06lX -> tios::%s", (unsigned long)addr, romcalls_get_name(rcid)); 
+
+	return buf;
+}
+
 uae_s32 ShowEA (FILE *f, int reg, amodes mode, wordsizes size, char *buf)
 {
     uae_u16 dp;
@@ -307,7 +315,7 @@ uae_s32 ShowEA (FILE *f, int reg, amodes mode, wordsizes size, char *buf)
     uae_u32 dispreg;
     uaecptr addr;
     uae_s32 offset = 0;
-    char buffer[80];
+    char buffer[296];
 
     switch (mode){
      case Dreg:
@@ -328,8 +336,10 @@ uae_s32 ShowEA (FILE *f, int reg, amodes mode, wordsizes size, char *buf)
      case Ad16:
 	disp16 = get_iword_1 (m68kpc_offset); m68kpc_offset += 2;
 	addr = m68k_areg(regs,reg) + (uae_s16)disp16;
-	sprintf (buffer,"(A%d,$%04x) == $%08lx", reg, disp16 & 0xffff,
-					(unsigned long)addr);
+	if (disp16 & 0x8000)
+		sprintf (buffer,"(-$%X,A%d) [%s]", (-(uae_s16)(disp16)) & 0xffff, reg, sym_addr ((uae_u32)addr));
+	else
+		sprintf (buffer,"(+$%X,A%d) [%s]", disp16 & 0xffff, reg, sym_addr ((uae_u32)addr));
 	break;
      case Ad8r:
 	dp = get_iword_1 (m68kpc_offset); m68kpc_offset += 2;
@@ -358,16 +368,15 @@ uae_s32 ShowEA (FILE *f, int reg, amodes mode, wordsizes size, char *buf)
 	    if (dp & 4) base += dispreg;
 
 	    addr = base + outer;
-	    sprintf (buffer,"(%s%c%d.%c*%d+%ld)+%ld == $%08lx", name,
+	    sprintf (buffer,"(%s%c%d.%c+%ld)+%ld [$%06lX]", name,
 		    dp & 0x8000 ? 'A' : 'D', (int)r, dp & 0x800 ? 'L' : 'W',
-		    1 << ((dp >> 9) & 3),
 		    disp,outer,
 		    (unsigned long)addr);
 	} else {
 	  addr = m68k_areg(regs,reg) + (uae_s32)((uae_s8)disp8) + dispreg;
-	  sprintf (buffer,"(A%d, %c%d.%c*%d, $%02x) == $%08lx", reg,
+	  sprintf (buffer,"(A%d, %c%d.%c, $%02X) [$%06lX]", reg,
 	       dp & 0x8000 ? 'A' : 'D', (int)r, dp & 0x800 ? 'L' : 'W',
-	       1 << ((dp >> 9) & 3), disp8,
+	       disp8,
 	       (unsigned long)addr);
 	}
 	break;
@@ -375,7 +384,10 @@ uae_s32 ShowEA (FILE *f, int reg, amodes mode, wordsizes size, char *buf)
 	addr = m68k_getpc () + m68kpc_offset;
 	disp16 = get_iword_1 (m68kpc_offset); m68kpc_offset += 2;
 	addr += (uae_s16)disp16;
-	sprintf (buffer,"(PC,$%04x) == $%08lx", disp16 & 0xffff,(unsigned long)addr);
+	if (disp16 & 0x8000)
+		sprintf (buffer,"(-$%X,PC) [%s]", (-(uae_s16)(disp16)) & 0xffff, sym_addr ((uae_u32)addr));
+	else
+		sprintf (buffer,"(+$%X,PC) [%s]", disp16 & 0xffff, sym_addr ((uae_u32)addr));
 	break;
      case PC8r:
 	addr = m68k_getpc () + m68kpc_offset;
@@ -405,38 +417,37 @@ uae_s32 ShowEA (FILE *f, int reg, amodes mode, wordsizes size, char *buf)
 	    if (dp & 4) base += dispreg;
 
 	    addr = base + outer;
-	    sprintf (buffer,"(%s%c%d.%c*%d+%ld)+%ld == $%08lx", name,
+	    sprintf (buffer,"(%s%c%d.%c+%ld)+%ld [%s]", name,
 		    dp & 0x8000 ? 'A' : 'D', (int)r, dp & 0x800 ? 'L' : 'W',
-		    1 << ((dp >> 9) & 3),
 		    disp,outer,
-		    (unsigned long)addr);
+		    sym_addr ((uae_u32)addr));
 	} else {
 	  addr += (uae_s32)((uae_s8)disp8) + dispreg;
-	  sprintf (buffer,"(PC, %c%d.%c*%d, $%02x) == $%08lx", dp & 0x8000 ? 'A' : 'D',
-		(int)r, dp & 0x800 ? 'L' : 'W',  1 << ((dp >> 9) & 3),
-		disp8, (unsigned long)addr);
+	  sprintf (buffer,"(PC, %c%d.%c, $%02X) [%s]", dp & 0x8000 ? 'A' : 'D',
+		(int)r, dp & 0x800 ? 'L' : 'W',
+		disp8, sym_addr ((uae_u32)addr));
 	}
 	break;
      case absw:
-	sprintf (buffer,"$%08lx", (unsigned long)(uae_s32)(uae_s16)get_iword_1 (m68kpc_offset));
+	sprintf (buffer,"$%lX", (unsigned long)(uae_s32)(uae_s16)get_iword_1 (m68kpc_offset));
 	m68kpc_offset += 2;
 	break;
      case absl:
-	sprintf (buffer,"$%08lx", (unsigned long)get_ilong_1 (m68kpc_offset));
+	sprintf (buffer,"%s", sym_addr ((uae_u32)get_ilong_1 (m68kpc_offset)));
 	m68kpc_offset += 4;
 	break;
      case imm:
 	switch (size){
 	 case sz_byte:
-	    sprintf (buffer,"#$%02x", (unsigned int)(get_iword_1 (m68kpc_offset) & 0xff));
+	    sprintf (buffer,"#$%X", (unsigned int)(get_iword_1 (m68kpc_offset) & 0xff));
 	    m68kpc_offset += 2;
 	    break;
 	 case sz_word:
-	    sprintf (buffer,"#$%04x", (unsigned int)(get_iword_1 (m68kpc_offset) & 0xffff));
+	    sprintf (buffer,"#$%X", (unsigned int)(get_iword_1 (m68kpc_offset) & 0xffff));
 	    m68kpc_offset += 2;
 	    break;
 	 case sz_long:
-	    sprintf (buffer,"#$%08lx", (unsigned long)(get_ilong_1 (m68kpc_offset)));
+	    sprintf (buffer,"#$%lX", (unsigned long)(get_ilong_1 (m68kpc_offset)));
 	    m68kpc_offset += 4;
 	    break;
 	 default:
@@ -446,21 +457,21 @@ uae_s32 ShowEA (FILE *f, int reg, amodes mode, wordsizes size, char *buf)
      case imm0:
 	offset = (uae_s32)(uae_s8)get_iword_1 (m68kpc_offset);
 	m68kpc_offset += 2;
-	sprintf (buffer,"#$%02x", (unsigned int)(offset & 0xff));
+	sprintf (buffer,"#$%X", (unsigned int)(offset & 0xff));
 	break;
      case imm1:
 	offset = (uae_s32)(uae_s16)get_iword_1 (m68kpc_offset);
 	m68kpc_offset += 2;
-	sprintf (buffer,"#$%04x", (unsigned int)(offset & 0xffff));
+	sprintf (buffer,"#$%X", (unsigned int)(offset & 0xffff));
 	break;
      case imm2:
 	offset = (uae_s32)get_ilong_1 (m68kpc_offset);
 	m68kpc_offset += 4;
-	sprintf (buffer,"#$%08lx", (unsigned long)offset);
+	sprintf (buffer,"#$%lX", (unsigned long)offset);
 	break;
      case immi:
 	offset = (uae_s32)(uae_s8)(reg & 0xff);
-	sprintf (buffer,"#$%08lx", (unsigned long)offset);
+	sprintf (buffer,"#$%lX", (unsigned long)offset);
 	break;
      default:
 	break;
@@ -471,8 +482,9 @@ uae_s32 ShowEA (FILE *f, int reg, amodes mode, wordsizes size, char *buf)
 	strcat (buf, buffer);
     return offset;
 }
-#endif /* 0 */
+#endif /* NO_GDB */
 
+#if 0
 /* The plan is that this will take over the job of exception 3 handling -
  * the CPU emulation functions will just do a longjmp to m68k_go whenever
  * they hit an odd address. */
@@ -484,7 +496,6 @@ static int verify_ea (int reg, amodes mode, wordsizes size, uae_u32 *val)
     int r;
     uae_u32 dispreg;
     uaecptr addr;
-    uae_s32 offset = 0;
 
     switch (mode){
      case Dreg:
@@ -598,6 +609,7 @@ static int verify_ea (int reg, amodes mode, wordsizes size, uae_u32 *val)
     last_instructionaccess_for_exception_3 = 0;
     return 0;
 }
+#endif /* 0 */
 
 uae_u32 get_disp_ea_020 (uae_u32 base, uae_u32 dp)
 {
@@ -831,7 +843,10 @@ void Exception_normal (int nr, uaecptr oldpc)
 		m68k_areg(regs, 7) -= 2;
 		put_word (m68k_areg(regs, 7), 0xb000 + nr * 4);
 	    }
+#if 0
 	    write_log ("Exception %d (%x) at %x -> %x!\n", nr, oldpc, currpc, get_long (regs.vbr + 4*nr));
+#endif
+	    
 	} else if (nr ==5 || nr == 6 || nr == 7 || nr == 9) {
 	    m68k_areg(regs, 7) -= 4;
 	    put_long (m68k_areg(regs, 7), oldpc);
@@ -877,7 +892,7 @@ kludge_me_do:
     if (newpc & 1) {
 #if 0
 	if (nr == 2 || nr == 3)
-	    uae_reset (); /* there is nothing else we can do.. */
+	    uae_reset (1); /* there is nothing else we can do.. */
 	else
 #endif /* 0 */
 	    exception3 (regs.ir, m68k_getpc(), newpc);
@@ -1091,7 +1106,7 @@ void m68k_divl (uae_u32 opcode, uae_u32 src, uae_u16 extra, uaecptr oldpc)
 	uae_s32 lo = (uae_s32)m68k_dreg(regs, (extra >> 12) & 7);
 	uae_s32 hi = lo < 0 ? -1 : 0;
 	uae_s32 save_high;
-	uae_u32 quot, rem;
+	uae_u32 quot = 0, rem = 0;
 	uae_u32 sign;
 
 	if (extra & 0x400) {
@@ -1250,12 +1265,24 @@ void m68k_mull (uae_u32 opcode, uae_u32 src, uae_u16 extra)
     }
 #endif
 }
+#ifdef NO_GDB
 static char* ccnames[] =
-{ "T ","F ","HI","LS","CC","CS","NE","EQ",
+{ "T","F","HI","LS","CC","CS","NE","EQ",
   "VC","VS","PL","MI","GE","LT","GT","LE" };
+#endif /* NO_GDB */
 
 void m68k_reset (void)
 {
+#if 0 /* The CPU is always the same in TiEmu. */
+    if (currprefs.cpu_level != changed_prefs.cpu_level
+	|| currprefs.address_space_24 != changed_prefs.address_space_24)
+    {
+	currprefs.address_space_24 = changed_prefs.address_space_24;
+	currprefs.cpu_level = changed_prefs.cpu_level;
+	build_cpufunctbl ();
+    }
+#endif /* 0 */
+
 //    regs.kick_mask = 0x00F80000;
     regs.spcflags = 0;
 #if 0
@@ -1294,9 +1321,9 @@ void m68k_reset (void)
 
 unsigned long REGPARAM2 op_illg (uae_u32 opcode)
 {
+#if 0
     uaecptr pc = m68k_getpc ();
     
-#if 0
     if (cloanto_rom && (opcode & 0xF100) == 0x7100) {
 	m68k_dreg (regs, (opcode >> 9) & 7) = (uae_s8)(opcode & 0xFF);
 	m68k_incpc (2);
@@ -1345,7 +1372,7 @@ unsigned long REGPARAM2 op_illg (uae_u32 opcode)
 	    call_calltrap (opcode & 0xFFF);
 	}
 #endif /* 0 */
-#if CYGNUS_SIM
+#if defined(CYGNUS_SIM) && !defined(NO_GDB)
 	/* Ignore ER_ASAP_TOO_LONG when running a file to debug. */
 	if (symfile && opcode == 0xA000 + 161) {
 		m68k_incpc(2);
@@ -1377,6 +1404,7 @@ void mmu_op(uae_u32 opcode, uae_u16 extra)
 	op_illg (opcode);
 }
 
+#if 0
 static uaecptr last_trace_ad = 0;
 
 static void do_trace (void)
@@ -1414,7 +1442,6 @@ static void do_trace (void)
     }
 }
 
-#if 0
 static int do_specialties (int cycles)
 {
     if (regs.spcflags & SPCFLAG_COPPER)
@@ -1434,14 +1461,14 @@ static int do_specialties (int cycles)
 	    do_copper ();
     }
 
-    if (regs.spcflags & SPCFLAG_DOTRACE) {
+    if (regs.spcflags & SPCFLAG_DOTRACE)
 	Exception (9,last_trace_ad);
-    }
+
     while (regs.spcflags & SPCFLAG_STOP) {
 	do_cycles (4 * CYCLE_UNIT);
 	if (regs.spcflags & SPCFLAG_COPPER)
 	    do_copper ();
-	if (regs.spcflags & (SPCFLAG_INT | SPCFLAG_DOINT)){
+	if (regs.spcflags & (SPCFLAG_INT | SPCFLAG_DOINT)) {
 	    int intr = intlev ();
 	    unset_special (SPCFLAG_INT | SPCFLAG_DOINT);
 	    if (intr != -1 && intr > regs.intmask) {
@@ -1592,14 +1619,13 @@ void m68k_go (int may_quit)
 		/* system is very badly confused */
 		write_log ("double bus error or corrupted stack, forcing reboot..\n");
 		regs.panic = 0;
-		uae_reset ();
+		uae_reset (1);
 	    }
 	}
-	m68k_run1 (currprefs.cpu_compatible ? m68k_run_1 : m68k_run_2);
+	m68k_run1 (currprefs.cpu_level == 0 ? m68k_run_1 : m68k_run_2);
     }
     in_m68k_go--;
 }
-#endif /* 0 */
 
 static void m68k_verify (uaecptr addr, uaecptr *nextpc)
 {
@@ -1628,25 +1654,32 @@ static void m68k_verify (uaecptr addr, uaecptr *nextpc)
 	}
     }
 }
+#endif /* 0 */
 
-#if 0
-void m68k_disasm (FILE *f, uaecptr addr, uaecptr *nextpc, int cnt)
+#ifdef NO_GDB
+int DasmFPU(uint16_t code, char *buf);
+
+int m68k_disasm (char *output, uaecptr addr)
 {
+    char buf[273];
     uaecptr newpc = 0;
-    m68kpc_offset = addr - m68k_getpc ();
-    while (cnt-- > 0) {
+
 	char instrname[20],*ccpt;
-	int opwords;
 	uae_u32 opcode;
 	struct mnemolookup *lookup;
 	struct instr *dp;
-	fprintf (f, "%08lx: ", m68k_getpc () + m68kpc_offset);
-	for (opwords = 0; opwords < 5; opwords++){
-	    fprintf (f, "%04x ", get_iword_1 (m68kpc_offset + opwords*2));
-	}
+	uae_u32 orig_opcode;
+	uaecptr nextpc;
+
+	m68kpc_offset = addr - m68k_getpc ();
+	output[0] = '\0';
+
+	sprintf (buf, "%06lx: ", m68k_getpc () + m68kpc_offset);
+	strcat (output, buf);
 	opcode = get_iword_1 (m68kpc_offset);
+	orig_opcode = opcode;
 	m68kpc_offset += 2;
-	if (cpufunctbl[opcode] == op_illg_1) {
+	if (cpufunctbl[opcode] == op_illg_1 || (orig_opcode & 0xf000) == 0xa000 || (orig_opcode & 0xf000) == 0xf000) {
 	    opcode = 0x4AFC;
 	}
 	dp = table68k + opcode;
@@ -1658,37 +1691,110 @@ void m68k_disasm (FILE *f, uaecptr addr, uaecptr *nextpc, int cnt)
 	if (ccpt != 0) {
 	    strncpy (ccpt, ccnames[dp->cc], 2);
 	}
-	fprintf (f, "%s", instrname);
-	switch (dp->size){
-	 case sz_byte: fprintf (f, ".B "); break;
-	 case sz_word: fprintf (f, ".W "); break;
-	 case sz_long: fprintf (f, ".L "); break;
-	 default: fprintf (f, "   "); break;
+	strcat (output, instrname);
+	switch (dp->size)
+	{
+	 case sz_byte: strcat (output, ".B "); break;
+	 case sz_word: strcat (output, ".W "); break;
+	 case sz_long: strcat (output, ".L "); break;
+	 default: strcat (output, "   "); break;
 	}
 
 	if (dp->suse) {
 	    newpc = m68k_getpc () + m68kpc_offset;
-	    newpc += ShowEA (f, dp->sreg, dp->smode, dp->size, 0);
+	    newpc += ShowEA (0, dp->sreg, dp->smode, dp->size, output);
 	}
 	if (dp->suse && dp->duse)
-	    fprintf (f, ",");
+	    strcat(output, ",");
 	if (dp->duse) {
 	    newpc = m68k_getpc () + m68kpc_offset;
-	    newpc += ShowEA (f, dp->dreg, dp->dmode, dp->size, 0);
+	    newpc += ShowEA (0, dp->dreg, dp->dmode, dp->size, output);
 	}
 	if (ccpt != 0) {
-	    if (cctrue(dp->cc))
-		fprintf (f, " == %08lx (TRUE)", newpc);
-	    else
-		fprintf (f, " == %08lx (FALSE)", newpc);
-	} else if ((opcode & 0xff00) == 0x6100) /* BSR */
-	    fprintf (f, " == %08lx", newpc);
-	fprintf (f, "\n");
-    }
-    if (nextpc)
-	*nextpc = m68k_getpc () + m68kpc_offset;
-}
+	    if (cctrue(dp->cc)) {
+		sprintf (buf, " [%06lX] (TRUE)", newpc);
+		strcat (output, buf);
+	    } else {
+		sprintf (buf, " [%06lX] (FALSE)", newpc);
+		strcat (output, buf);
+	    }
+	} else if ((opcode & 0xff00) == 0x6100) { /* BSR */
+	    sprintf (buf, " [%s]", sym_addr (newpc));
+	    strcat (output, buf);
+	} else if((orig_opcode >= 0xf800) && (orig_opcode <= 0xfff2)) {
+		char *buffer = &(output[8]);
+		unsigned long pm;
+		uint32_t pc = m68k_getpc();	/* addr */
 
+		/* F-Line ROM calls (see KerNO doc and thanks to Lionel Debroux) */
+		switch(orig_opcode)
+		{
+		case 0xfff0:	/* 6 byte bsr w/long word displacement */
+			pm = get_ilong_1 (m68kpc_offset); m68kpc_offset += 6 - 2;
+			if (pm & 0x8000)
+				sprintf (buffer, "FLINE bsr.l *-$%lX [%lX]", (-(signed long)(int32_t)pm) - 2, pc + (signed long)(int32_t)pm + 2);
+			else
+				sprintf (buffer, "FLINE bsr.l *+$%lX [%lX]", pm + 2, pc + pm + 2);
+			break;
+		case 0xfff1:	/* 6 byte bra w/long word displacement */
+			pm = get_ilong_1 (m68kpc_offset); m68kpc_offset += 6 - 2;
+			if (pm & 0x8000)
+				sprintf (buffer, "FLINE bra.l *-$%lX [%lX]", (-(signed long)(int32_t)pm) - 2, pc + (signed long)(int32_t)pm + 2);
+			else
+				sprintf (buffer, "FLINE bra.l *+$%lX [%lX]", pm + 2, pc + pm + 2);
+			break;
+		case 0xfff2:	/* 4 byte ROM CALL */
+			pm = get_iword_1 (m68kpc_offset); m68kpc_offset += 4 - 2;
+			sprintf (buffer, "FLINE $%04x.l [%s]", pm/4, romcalls_get_name(pm / 4));
+			break;
+		case 0xffee:	/* jmp __ld_entry_point_plus_0x8000+word */
+			pm = get_iword_1 (m68kpc_offset); m68kpc_offset += 4 - 2;
+			{
+				int handle;
+				uint32_t addr;
+				
+				heap_search_for_address(pc + 2, &handle);
+				if (handle > 0) heap_get_block_addr(handle, &addr); else addr = 0;
+				sprintf (buffer, "FLINE jmp.w *+$%lX [%lX]", (signed long)(signed short)pm + 0x8000, addr + (signed long)(signed short)pm + 0x8000);
+			}
+			break;
+		case 0xffef:	/* jsr __ld_entry_point_plus_0x8000+word */
+			pm = get_iword_1 (m68kpc_offset); m68kpc_offset += 4 - 2;
+			{
+				int handle;
+				uint32_t addr;
+				
+				heap_search_for_address(pc + 2, &handle);
+				if (handle > 0) heap_get_block_addr(handle, &addr); else addr = 0;
+				sprintf (buffer, "FLINE jsr.w *+$%lX [%lX]", (signed long)(signed short)pm + 0x8000, addr + (signed long)(signed short)pm + 0x8000);
+			}
+			break;
+		case 0xf8b5:	/* 2 byte ROM call followed by an FPU opcode (special case: _bcd_math) */
+			{
+				char tmp[64];
+				pm = get_iword_1 (m68kpc_offset); m68kpc_offset += 4 - 2;
+				DasmFPU(pm, tmp);
+				sprintf (buffer, "FLINE _bcd_math (FPU: %s)", tmp);
+				break;
+			}
+		default:		/* 2 byte ROM CALL */
+			sprintf (buffer, "FLINE $%03x.w [%s]", opcode & 0x7ff, romcalls_get_name(opcode & 0x7ff));
+			break;
+		}
+	} else if ((orig_opcode & 0xf000) == 0xa000) { /* ER_throw */
+		char *buffer = &(output[8]);
+		sprintf (buffer, "ER_throw %d [%s]", opcode & 0xfff, ercodes_get_name(opcode & 0xfff));
+	} else if (opcode == 0x4AFC && orig_opcode != 0x4AFC) { /* illegal instruction, but not ILLEGAL */
+		sprintf (output, "%06lx: DC.W $%04X", addr, orig_opcode);
+		
+	}
+
+	nextpc = m68k_getpc () + m68kpc_offset;	
+	return (nextpc - addr);
+}
+#endif /* NO_GDB */
+
+#if 0
 void m68k_dumpstate (FILE *f, uaecptr *nextpc)
 {
     int i;
@@ -1786,11 +1892,14 @@ uae_u8 *restore_cpu (uae_u8 *src)
 
 static int cpumodel[] = { 68000, 68010, 68020, 68020 };
 
-uae_u8 *save_cpu (int *len)
+uae_u8 *save_cpu (int *len, uae_u8 *dstptr)
 {
     uae_u8 *dstbak,*dst;
     int model,i;
 
+    if (dstptr)
+	dstbak = dst = dstptr;
+    else
     dstbak = dst = malloc(4+4+15*4+4+4+4+4+2+4+4+4+4+4+4+4);
     model = cpumodel[currprefs.cpu_level];
     save_u32 (model);					/* MODEL */
