@@ -1,13 +1,13 @@
 /* Hey EMACS -*- linux-c -*- */
-/* $Id: popup.c 1676 2005-08-22 20:51:30Z roms $ */
+/* $Id: popup.c 2750 2007-12-30 21:34:47Z kevinkofler $ */
 
-/*  TiEmu - an TI emulator
+/*  TiEmu - Tiemu Is an EMUlator
  *
  *  Copyright (c) 2000-2001, Thomas Corvazier, Romain Lievin
  *  Copyright (c) 2001-2003, Romain Lievin
  *  Copyright (c) 2003, Julien Blache
  *  Copyright (c) 2004, Romain Livin
- *  Copyright (c) 2005, Romain Livin, Kevin Kofler
+ *  Copyright (c) 2005-2007, Romain Livin, Kevin Kofler
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -26,6 +26,11 @@
 #  include <config.h>
 #endif
 
+/* GTK+ 2.12 changed the tooltip API, see:
+   http://library.gnome.org/devel/gtk/2.12/gtk-migrating-tooltips.html
+   This means GtkTooltipsData is now deprecated. */
+#undef GTK_DISABLE_DEPRECATED
+
 #include <gtk/gtk.h>
 #include <glade/glade.h>
 #include <stdlib.h>
@@ -40,9 +45,9 @@
 #include "version.h"
 #include "popup.h"
 #include "paths.h"
-#include "../engine.h"
+#include "engine.h"
 #include "fs_misc.h"
-#include "comm.h"
+#include "device.h"
 #include "rcfile.h"
 #include "dboxes.h"
 #include "romversion.h"
@@ -54,6 +59,12 @@
 #include "scroptions.h"
 #include "tie_error.h"
 #include "dbg_all.h"
+#include "quicksend.h"
+#include "filesel.h"
+#include "keypress.h"
+#ifndef NO_SOUND
+#include "audio.h"
+#endif
 
 #include "ti68k_int.h"
 #include "ti68k_def.h"
@@ -62,18 +73,12 @@ GLADE_CB void
 on_popup_menu_header                   (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
-#if 1
-	fprintf(stdout, _("* TiEmu version %s (cables=%s, files=%s, calcs=%s)\n"),
-	     TIEMU_VERSION, ticable_get_version(), tifiles_get_version(),
-	     ticalc_get_version());
-#else
-	gchar **list, **ptr;
-
-	list = create_fsels(inst_paths.base_dir, NULL, "*.txt");
-	for(ptr = list; *ptr; ptr++)	printf("<<%s>>\n", *ptr);
-#endif
+	fprintf(stdout, _("* TiEmu version %s (cables=%s, files=%s, calcs=%s, conv=%s)"),
+	     TIEMU_VERSION, 
+		 ticables_version_get(), tifiles_version_get(), ticalcs_version_get(), ticonv_version_get());
 }
 
+/* menu part 1 (link) */
 
 GLADE_CB void
 on_send_file_to_tiemu1_activate     (GtkMenuItem     *menuitem,
@@ -82,7 +87,12 @@ on_send_file_to_tiemu1_activate     (GtkMenuItem     *menuitem,
 	if(engine_is_stopped()) return;
 
 	engine_stop();
-	display_send_files_dbox();
+
+	if(!options.qs_enabled)
+		display_send_files_dbox();
+	else if(options.qs_enabled && options.qs_file && strlen(options.qs_file))
+		fs_send_file(options.qs_file);
+
 	engine_start();
 }
 
@@ -90,8 +100,44 @@ GLADE_CB void
 on_recv_file_from_tiemu1_activate     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
 {
+	int active;
 	if(engine_is_stopped()) return;
-	params.recv_file = GTK_CHECK_MENU_ITEM(menuitem)->active;
+	active = GTK_CHECK_MENU_ITEM(menuitem)->active;
+#ifndef NO_SOUND
+	if (active)
+		audio_disable();
+#endif
+	params.recv_file = active;
+}
+
+GLADE_CB void
+on_emulate_sound1_activate     (GtkMenuItem     *menuitem,
+                                gpointer         user_data)
+{
+#ifndef NO_SOUND
+	int active;
+	if(engine_is_stopped()) return;
+	active = GTK_CHECK_MENU_ITEM(menuitem)->active;
+	if (active) {
+		params.recv_file = 0;
+		audio_enable();
+	} else
+		audio_disable();
+#endif
+}
+
+
+GLADE_CB void
+on_debug_file_with_tiemu1_activate     (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+	if(engine_is_stopped()) return;
+
+	engine_stop();
+#ifndef NO_GDB
+	display_debug_dbox();
+#endif
+	engine_start();
 }
 
 
@@ -102,10 +148,22 @@ on_link_cable1_activate                (GtkMenuItem     *menuitem,
 	if(engine_is_stopped()) return;
 
 	engine_stop();
-	display_comm_dbox();
+	display_device_dbox();
 	engine_start();
 }
 
+GLADE_CB void
+on_quick_send1_activate                (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+	if(engine_is_stopped()) return;
+
+	engine_stop();
+	display_quicksend_dbox();
+	engine_start();
+}
+
+/* menu part 2 (config & state) */
 
 void window_get_rect(GtkWidget *widget, GdkRect *rect);
 
@@ -120,10 +178,10 @@ on_save_config1_activate               (GtkMenuItem     *menuitem,
 	rcfile_write();
 
 #ifdef __WIN32__
-  	msg_box(_("Information"), 
+  	msg_box1(_("Information"), 
 	  _("Configuration file saved (in tiemu.ini)."));
 #else
-	msg_box(_("Information"), 
+	msg_box1(_("Information"), 
 	  _("Configuration file saved (in ~/.tiemu)."));
 #endif
 }
@@ -135,7 +193,7 @@ on_load_config1_activate               (GtkMenuItem     *menuitem,
 {
 	rcfile_read();
 
-  	msg_box(_("Information"), _("Configuration file loaded."));
+  	msg_box1(_("Information"), _("Configuration file loaded."));
 }
 
 
@@ -168,6 +226,7 @@ on_revert_to_saved_state1_activate     (GtkMenuItem     *menuitem,
   	engine_start();
 }
 
+/* menu part 3 (debug) */
 
 GLADE_CB void
 on_enter_debugger1_activate            (GtkMenuItem     *menuitem,
@@ -182,27 +241,27 @@ on_enter_debugger1_activate            (GtkMenuItem     *menuitem,
 #endif
 }
 
-GLADE_CB void
-on_quit1_activate                      (GtkMenuItem     *menuitem,
-                                        gpointer         user_data);
-
 GLADE_CB void
 on_reset_calc1_activate                (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
 	engine_stop();
-#if 1
-	if(msg_box2("Question", "Clear RAM ?") == BUTTON1)
-		memset(tihw.ram, 0, tihw.ram_size);
-#endif
-
-	ti68k_reset();
-	if (dbg_on)
-		on_quit1_activate(menuitem, user_data);
-	else
-	  	engine_start();
+	switch (msg_box3(_("Question"), _("Clear RAM?"), GTK_STOCK_YES, GTK_STOCK_NO, GTK_STOCK_CANCEL)) {
+	  case BUTTON1:
+	    memset(tihw.ram, 0, tihw.ram_size);
+	  case BUTTON2:
+	    ti68k_reset();
+	    if (dbg_on)
+	      gtk_debugger_close();
+	    else {
+	      default:
+	        engine_start();
+	    }
+	}
 }
 
+/* menu part 4 (images) */
+
 GLADE_CB void
 on_upgrade_calc1_activate                (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
@@ -222,6 +281,7 @@ on_set_rom1_activate                   (GtkMenuItem     *menuitem,
 	engine_start();
 }
 
+/* menu part 5.1 (emulator options) */
 
 GLADE_CB void
 on_restrict_to_actual_speed1_activate  (GtkMenuItem     *menuitem,
@@ -302,6 +362,7 @@ on_custom_view1_activate               (GtkMenuItem     *menuitem,
 {
 }
 
+/* menu part 5.2 (skin options) */
 
 GLADE_CB void
 on_no_skin1_activate                      (GtkMenuItem     *menuitem,
@@ -330,12 +391,16 @@ on_set_skin1_activate                  (GtkMenuItem     *menuitem,
 	display_skin_dbox();
 }
 
+/* menu part 5.3 (screenshot options) */
 
 GLADE_CB void
 on_now1_activate                     (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
-	hid_screenshot(NULL);
+	if(options2.shots > 1)
+		hid_screenshot_burst();
+	else
+		hid_screenshot_single();
 }
 
 
@@ -346,6 +411,80 @@ on_screen_options1_activate               (GtkMenuItem     *menuitem,
 	display_scroptions_dbox();
 }
 
+GLADE_CB void
+on_copy_to_clipboard1_activate        (GtkMenuItem     *menuitem,
+                                       gpointer         user_data)
+{
+	options2.clipboard = GTK_CHECK_MENU_ITEM(menuitem)->active ? 1 : 0;
+}
+
+/* menu part 5.4 (key press options) */
+
+GLADE_CB void
+on_setup_recording1_activate             (GtkMenuItem     *menuitem,
+                                          gpointer         user_data)
+{
+	const gchar *filename = create_fsel(inst_paths.base_dir, "keypress.txt", "*.txt", TRUE);
+	if (!filename)
+		return;
+
+	g_free(options.kp_rec_file);
+	options.kp_rec_file = g_strdup(filename);
+}
+
+GLADE_CB void
+on_start_recording1_activate             (GtkMenuItem     *menuitem,
+                                          gpointer         user_data)
+{
+	if(options.kp_ply_enabled || !options.kp_rec_file) return;
+
+	options.kp_rec_enabled = !options.kp_rec_enabled;
+
+	if(options.kp_rec_enabled)
+		kp_recording_start(options.kp_rec_file);
+	else
+		kp_recording_stop();
+}
+
+GLADE_CB void
+on_setup_playing1_activate             (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+	const gchar *filename = create_fsel(inst_paths.base_dir, NULL, "*.txt", FALSE);
+	if (!filename)
+		return;
+
+	g_free(options.kp_ply_file);
+	options.kp_ply_file = g_strdup(filename);
+}
+
+static gboolean kp_callback(gpointer data)
+{
+	int key, action, ret;
+
+	ret = kp_playing_key(&key, &action);
+	if(ret) 
+	{
+		kp_playing_stop();
+		return FALSE;
+	}
+
+	ti68k_kbd_set_key(key, action);
+	//printf("%i %i\n", key, action);
+
+	return TRUE;
+}
+
+GLADE_CB void
+on_start_playing1_activate             (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+	kp_playing_start(options.kp_ply_file);
+	g_timeout_add(250, kp_callback, NULL);
+}
+
+/* menu part 6 (misc) */
+
 static void go_to_bookmark(const char *link);
 
 GLADE_CB void
@@ -370,7 +509,7 @@ GLADE_CB void
 on_bookmarks1_activate				   (GtkMenuItem		*menuitem,
 										gpointer		user_data)
 {
-	GtkTooltipsData* data = gtk_tooltips_data_get(GTK_WIDGET(menuitem));
+	GtkTooltipsData* data = gtk_tooltips_data_get(GTK_WIDGET(menuitem)); /* FIXME: deprecated in GTK+ 2.12 */
 	go_to_bookmark(data->tip_text);
 }
 
@@ -381,7 +520,7 @@ on_bugreport1_activate				   (GtkMenuItem     *menuitem,
 {
 	GtkWidget *dialog;
 	const gchar *message =
-    "There are several ways to get in touch if you encounter a problem with TiEMu or if you have questions, suggestions, bug reports, etc:\n- if you have general questions or problems, please consider the users' mailing list first (http://tiemu-users@list.sf.net).\n- if you want to discuss about TiEmu, you can use the TiEmu forum (http://sourceforge.net/forum/?group_id=23169).\n- for bug reports, use the 'Bug Tracking System' (http://sourceforge.net/tracker/?group_id=23169).\n\nBefore e-mailing the TiEmu team, make sure you have read the manual and/or the FAQ....";
+    "There are several ways to get in touch if you encounter a problem with TiEmu or if you have questions, suggestions, bug reports, etc:\n- if you have general questions or problems, please consider the users' mailing list first (http://tiemu-users@list.sf.net).\n- if you want to discuss about TiEmu, you can use the TiEmu forum (http://sourceforge.net/forum/?group_id=23169).\n- for bug reports, use the 'Bug Tracking System' (http://sourceforge.net/tracker/?group_id=23169).\n\nBefore e-mailing the TiEmu team, make sure you have read the manual and/or the FAQ....";
   
 	dialog = gtk_message_dialog_new(NULL, GTK_DIALOG_MODAL,
 				  GTK_MESSAGE_INFO, GTK_BUTTONS_CLOSE,
@@ -433,7 +572,7 @@ on_exit_and_save_state1_activate                      (GtkMenuItem     *menuitem
 
 	// set path
 	g_free(params.sav_file);
-	params.sav_file = g_strconcat(inst_paths.home_dir, basename, ".sav", NULL);
+	params.sav_file = g_strconcat(inst_paths.img_dir, basename, ".sav", NULL);
 	g_free(basename);
 
 	// save state
@@ -480,6 +619,7 @@ on_exit_without_saving_state1_activate (GtkMenuItem     *menuitem,
 }
 
 typedef void (*VCB) (void);
+extern int reset_disabled;
 
 /*
   Display the GTK popup menu and configure some items
@@ -496,7 +636,7 @@ GtkWidget* display_popup_menu(void)
 	    (tilp_paths_build_glade("popup-2.glade"), "popup_menu",
 	     PACKAGE);
 	if (!xml)
-		g_error(_("%s: GUI loading failed !\n"), __FILE__);
+		g_error(_("%s: GUI loading failed!\n"), __FILE__);
 	glade_xml_signal_autoconnect(xml);
 
 	menu = glade_xml_get_widget(xml, "popup_menu");
@@ -510,8 +650,14 @@ GtkWidget* display_popup_menu(void)
 	// init check buttons
 	data = glade_xml_get_widget(xml, "recv_file_from_tiemu1");
 	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(data), params.recv_file);
+	data = glade_xml_get_widget(xml, "emulate_sound1");
+#ifdef NO_SOUND
+	gtk_widget_set_sensitive(data, FALSE);
+#else
+	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(data), audio_isactive);
+#endif
 
-    data = glade_xml_get_widget(xml, "restrict1");
+	data = glade_xml_get_widget(xml, "restrict1");
 	g_signal_handlers_block_by_func(GTK_OBJECT(data), (VCB)on_restrict_to_actual_speed1_activate, NULL);
 	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(data), params.restricted);
 	g_signal_handlers_unblock_by_func(GTK_OBJECT(data), (VCB)on_restrict_to_actual_speed1_activate, NULL);
@@ -527,6 +673,11 @@ GtkWidget* display_popup_menu(void)
 	data = glade_xml_get_widget(xml, "full_view1");
 	gtk_widget_set_sensitive(data, FALSE);
 
+#ifdef NO_GDB
+	data = glade_xml_get_widget(xml, "debug_file_with_tiemu1");
+	gtk_widget_set_sensitive(data, FALSE);
+#endif
+
 	// init radio buttons
     switch(options.view)
     {
@@ -555,7 +706,7 @@ GtkWidget* display_popup_menu(void)
 		data = glade_xml_get_widget(xml, "high_lcd_update1");
         gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(data), TRUE);
 	}
-	else if(params.lcd_rate >= 40 && params.lcd_rate < 70 || params.lcd_rate == -1)
+	else if((params.lcd_rate >= 40 && params.lcd_rate < 70) || params.lcd_rate == -1)
 	{
 		data = glade_xml_get_widget(xml, "med_lcd_update1");
         gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(data), TRUE);
@@ -569,10 +720,15 @@ GtkWidget* display_popup_menu(void)
 	// if debugger is open, blocks some items
 	if(dbg_on)
 	{
+#ifndef NO_GDB
 		data = glade_xml_get_widget(xml, "send_file_to_tiemu1");
 		gtk_widget_set_sensitive(data, FALSE);
 
-		data = glade_xml_get_widget(xml, "recv_file_to_tiemu1");
+		data = glade_xml_get_widget(xml, "debug_file_with_tiemu1");
+		gtk_widget_set_sensitive(data, FALSE);
+#endif
+
+		data = glade_xml_get_widget(xml, "recv_file_from_tiemu1");
 		gtk_widget_set_sensitive(data, FALSE);
 
 		data = glade_xml_get_widget(xml, "link_cable1");
@@ -583,8 +739,30 @@ GtkWidget* display_popup_menu(void)
 
 		data = glade_xml_get_widget(xml, "set_rom1");
 		gtk_widget_set_sensitive(data, FALSE);
+
+		if (reset_disabled)
+		{
+			data = glade_xml_get_widget(xml, "reset_calc1");
+			gtk_widget_set_sensitive(data, FALSE);
+		}
+
+		data = glade_xml_get_widget(xml, "calculator_state1");
+		gtk_widget_set_sensitive(data, FALSE);
 	}
 
+	data = glade_xml_get_widget(xml, "start_recording1");
+	g_signal_handlers_block_by_func(GTK_OBJECT(data), (VCB)on_start_recording1_activate, NULL);
+	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(data), options.kp_rec_enabled);
+	g_signal_handlers_unblock_by_func(GTK_OBJECT(data), (VCB)on_start_recording1_activate, NULL);
+
+	data = glade_xml_get_widget(xml, "start_playing1");
+	g_signal_handlers_block_by_func(GTK_OBJECT(data), (VCB)on_start_playing1_activate, NULL);
+	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(data), options.kp_ply_enabled);
+	g_signal_handlers_unblock_by_func(GTK_OBJECT(data), (VCB)on_start_playing1_activate, NULL);
+
+	data = glade_xml_get_widget(xml, "copy_to_clipboard1");
+	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(data), options2.clipboard);
+
 	return menu;
 }
 
@@ -600,27 +778,35 @@ static void go_to_bookmark(const char *link)
 	hInst = ShellExecute(NULL, "open", link, NULL, NULL, SW_SHOWNORMAL);
 	if((int)hInst <= 32)
 	{
-		msg_box("Error", "Unable to run ShellExecture extension.");
+		msg_box1("Error", "Unable to run ShellExecture extension.");
 	}
 #else
 	// Kevin's list:
-	// * /usr/bin/gnome-open (GNOME 2.6+ default browser, this really should be
-	// first on the list to try, as this will honor the user's choice rather than
-	// guessing an arbitrary one)
+	// These ones should be first, as they will honor the user's choice rather
+	// than guessing an arbitrary one:
+	// * /usr/bin/xdg-open (runs the default browser of the desktop environment
+	// currently in use, this is the best solution)
+	// * /usr/bin/gnome-open (GNOME 2.6+ default browser, user-configurable)
+	// Distro-specific browser wrapper scripts:
 	// * /usr/bin/sensible-browser (Debian's browser script)
-	// * /usr/bin/htmlview (old RHL/Fedora default browser script)
+	// * /usr/bin/htmlview (old RHL/Fedora default browser script, current
+	// versions will honor the GNOME browser preference)
+	// Fallback to a hardcoded list of browsers:
 	// * /usr/bin/firefox (Mozilla Firefox)
-	// * /usr/bin/mozilla (Mozilla Seamonkey)
+	// * /usr/bin/seamonkey (Seamonkey)
 	// * /usr/bin/konqueror (Konqueror)
+	// * /usr/bin/mozilla (old Mozilla Suite)
 	//
 	gboolean result;
 	char *apps[] = { 
+			"/usr/bin/xdg-open",
 			"/usr/bin/gnome-open",
 			"/usr/bin/sensible-browser",
 			"/usr/bin/htmlview",
 			"/usr/bin/firefox",
-			"/usr/bin/mozilla",
+			"/usr/bin/seamonkey",
 			"/usr/bin/konqueror",
+			"/usr/bin/mozilla",
 	};
 	gint i, n;
 
@@ -642,7 +828,7 @@ static void go_to_bookmark(const char *link)
 
 	if (i == n) 
 	{
-		msg_box("Error", "Spawn error: do you have Mozilla installed ?");
+		msg_box1("Error", "Spawn error: do you have Firefox installed ?");
 	} 
 #endif
 	else 
