@@ -1,13 +1,13 @@
 /* Hey EMACS -*- linux-c -*- */
-/* $Id: dbg_code.c 1632 2005-08-04 20:32:33Z roms $ */
+/* $Id: dbg_code.c 2753 2007-12-30 23:14:15Z kevinkofler $ */
 
-/*  TiEmu - an TI emulator
+/*  TiEmu - Tiemu Is an EMUlator
  *
  *  Copyright (c) 2000-2001, Thomas Corvazier, Romain Lievin
  *  Copyright (c) 2001-2003, Romain Lievin
  *  Copyright (c) 2003, Julien Blache
  *  Copyright (c) 2004, Romain Livin
- *  Copyright (c) 2005, Romain Livin, Kevin Kofler
+ *  Copyright (c) 2005-2007, Romain Livin, Kevin Kofler
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -38,24 +38,28 @@
 #include "support.h"
 #include "ti68k_int.h"
 #include "struct.h"
+#include "dbg_wnds.h"
 #include "dbg_all.h"
 #include "dbg_romcall.h"
+#include "engine.h"
+#include "logger/log_link.h"
+
+gint reset_disabled = FALSE;
 
 //#define FIXED_SIZE
+//#define RUN_DBG_OPEN
 
 enum { 
 	    COL_ICON, COL_ADDR, COL_OPCODE, COL_OPERAND,
-        COL_HEXADDR, COL_FONT, COL_COLOR
+        COL_HEXADDR, COL_FONT,
 };
 #define CLIST_NVCOLS	(4)		// 4 visible columns
-#define CLIST_NCOLS		(7)		// 7 real columns
-
-#define FONT_NAME	"courier"
+#define CLIST_NCOLS		(6)		// 7 real columns
 
 #ifdef FIXED_SIZE
 #define NLINES      10
 #else
-static gint NLINES = 10;	
+static gint NLINES = 10;
 #endif
 
 GladeXML *xml = NULL;
@@ -67,13 +71,13 @@ static GtkListStore* clist_create(GtkWidget *widget)
 	GtkTreeModel *model;
 	GtkCellRenderer *renderer;
 	GtkTreeSelection *selection;
-    const gchar *text[CLIST_NVCOLS] = { _(""), _("Address"), _("Opcode"), _("Operand") };
+    const gchar *text[CLIST_NVCOLS] = { "  ", _("Address"), _("Opcode"), _("Operand") };
     gint i;
 	
 	store = gtk_list_store_new(CLIST_NCOLS,
 				GDK_TYPE_PIXBUF, 
                 G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING,
-                G_TYPE_INT, G_TYPE_STRING, GDK_TYPE_COLOR,
+                G_TYPE_INT, G_TYPE_STRING,
 				-1
             );
     model = GTK_TREE_MODEL(store);
@@ -83,6 +87,7 @@ static GtkListStore* clist_create(GtkWidget *widget)
 	gtk_tree_view_set_rules_hint(view, FALSE);
   
   	renderer = gtk_cell_renderer_pixbuf_new();
+	set_renderer_pad(renderer);
 	gtk_tree_view_insert_column_with_attributes(view, -1, 
             text[COL_ICON], renderer, 
             "pixbuf", COL_ICON,
@@ -91,10 +96,11 @@ static GtkListStore* clist_create(GtkWidget *widget)
 	for(i = COL_ADDR; i <= COL_OPERAND; i++)
 	{
 		renderer = gtk_cell_renderer_text_new();
+		set_renderer_pad(renderer);
 		gtk_tree_view_insert_column_with_attributes(view, -1, 
             text[i], renderer, 
             "text", i,
-			"background-gdk", COL_COLOR,
+			"font", COL_FONT,
 			NULL);
 	}
     
@@ -104,6 +110,12 @@ static GtkListStore* clist_create(GtkWidget *widget)
 		
 		col = gtk_tree_view_get_column(view, i);
 		gtk_tree_view_column_set_resizable(col, TRUE);
+#if 0
+		gtk_tree_view_column_set_resizable(col, FALSE);
+		gtk_tree_view_column_set_sizing (col, GTK_TREE_VIEW_COLUMN_FIXED);
+		gtk_tree_view_column_set_fixed_width(col, 100);
+		gtk_tree_view_set_fixed_height_mode(view, TRUE);		
+#endif
 	}
 	
 	selection = gtk_tree_view_get_selection(view);
@@ -117,8 +129,6 @@ static void clist_populate(GtkListStore *store, uint32_t addr)
     GtkTreeIter iter;
     GdkPixbuf *pix;
     gint i;
-    GdkColor *color, color1, color2;
-	gboolean success;
     uint32_t pc = ti68k_debug_get_pc();
 
 	/* We can't start disassembling stuff before GDB is even loaded, and in the
@@ -126,11 +136,6 @@ static void clist_populate(GtkListStore *store, uint32_t addr)
 	   of time. */
 	if (!dbg_on) return;
 
-	gdk_color_parse("White", &color1);
-	gdk_colormap_alloc_colors(gdk_colormap_get_system(), &color1, 1, FALSE, FALSE, &success);
-	gdk_color_parse("Green", &color2);
-	gdk_colormap_alloc_colors(gdk_colormap_get_system(), &color2, 1, FALSE, FALSE, &success);
-
     for(i = 0; i < NLINES; i++)
     {
         gchar *output;
@@ -157,12 +162,16 @@ static void clist_populate(GtkListStore *store, uint32_t addr)
 		else
 			row_text[2] = g_strdup(split[2]);
 
-		if(g_list_find(bkpts.code, GINT_TO_POINTER(addr)) != NULL)
-                pix = create_pixbuf("bkpt.xpm");
+		if((g_list_find(bkpts.code, GINT_TO_POINTER(addr)) != NULL) && (addr != pc) ||
+			(g_list_find(bkpts.code, GINT_TO_POINTER(addr | BKPT_TMP_MASK)) != NULL) && (addr != pc))
+            pix = create_pixbuf("bkpt.xpm");
+		else if((g_list_find(bkpts.code, GINT_TO_POINTER(addr)) != NULL) && (addr == pc) ||
+			(g_list_find(bkpts.code, GINT_TO_POINTER(addr | BKPT_TMP_MASK)) != NULL) && (addr == pc))
+			pix = create_pixbuf("run_2.xpm");
+		else if(addr == pc)
+			pix = create_pixbuf("run_1.xpm");
         else
-                pix = create_pixbuf("void.xpm");
-
-        color = (addr == pc) ? &color2 : &color1;
+            pix = create_pixbuf("void.xpm");
 
         gtk_list_store_append(store, &iter);
 	    gtk_list_store_set(store, &iter, 
@@ -171,10 +180,11 @@ static void clist_populate(GtkListStore *store, uint32_t addr)
 		COL_OPCODE, row_text[1],
         COL_OPERAND, row_text[2],
         COL_HEXADDR, value,
-		COL_FONT, FONT_NAME,
-		COL_COLOR, color,
 		-1);
 
+		if(options3.dbg_font_type)
+			gtk_list_store_set(store, &iter, COL_FONT, options3.dbg_font_name, -1);
+
         addr += offset;
         g_strfreev(split);
 		g_strfreev(row_text);
@@ -205,22 +215,16 @@ static void clist_refresh(GtkListStore *store, gboolean reload)
     GtkTreeIter iter;
 
     uint32_t addr;
-    uint32_t pc;
+    uint32_t pc, old_pc;
     int found = 0;
 
-	GdkColor *color, color1, color2;
-	gboolean success;
-
     const int offset = 3;   // 3 instructions are still visible
     uint32_t addr3;
     gint i;
 
-    pc = ti68k_debug_get_pc();
-
-	gdk_color_parse("White", &color1);
-	gdk_colormap_alloc_colors(gdk_colormap_get_system(), &color1, 1, FALSE, FALSE, &success);
-	gdk_color_parse("Green", &color2);
-	gdk_colormap_alloc_colors(gdk_colormap_get_system(), &color2, 1, FALSE, FALSE, &success);
+	// Data/Bit bkpt encounter after instruction execution so take care of this
+	addr3 = pc = ti68k_debug_get_pc();
+	old_pc = ti68k_debug_get_old_pc();
 
     // check for refresh (search for pc)
     for(valid = gtk_tree_model_get_iter_first(model, &iter), i = 0;
@@ -245,7 +249,8 @@ static void clist_refresh(GtkListStore *store, gboolean reload)
     if(!found && reload)
     {
         gtk_list_store_clear(store);
-        clist_populate(store, ti68k_debug_get_pc());
+		//printf("%06x %06x %i\n", old_pc, pc, abs(old_pc - pc));
+        clist_populate(store, abs((old_pc - pc)) > 8 ? pc : old_pc);
     }
 
     // repopulate so that 3 instructions are still visible at the bottom of the list
@@ -269,16 +274,18 @@ static void clist_refresh(GtkListStore *store, gboolean reload)
         gtk_tree_model_get(model, &iter, COL_ADDR, &str, -1);
         sscanf(str, "%x", &addr);
 
-		color = (addr == pc) ? &color2 : &color1;
-
-        if(g_list_find(bkpts.code, GINT_TO_POINTER(addr)) != NULL)
+        if(((g_list_find(bkpts.code, GINT_TO_POINTER(addr)) != NULL) && (addr != pc) ||
+			(g_list_find(bkpts.code, GINT_TO_POINTER(addr | BKPT_TMP_MASK)) != NULL) && (addr != pc)))
             pix = create_pixbuf("bkpt.xpm");
-		else if(g_list_find(bkpts.code, GINT_TO_POINTER(addr|BKPT_TMP_MASK)) != NULL)
-            pix = create_pixbuf("bkpt_tmp.xpm");
+		else if(((g_list_find(bkpts.code, GINT_TO_POINTER(addr)) != NULL) && (addr == pc) ||
+			(g_list_find(bkpts.code, GINT_TO_POINTER(addr | BKPT_TMP_MASK)) != NULL) && (addr == pc)))
+            pix = create_pixbuf("run_2.xpm");
+		else if(addr == pc)
+			pix = create_pixbuf("run_1.xpm");
         else
             pix = create_pixbuf("void.xpm");
 
-        gtk_list_store_set(store, &iter, COL_ICON, pix, COL_COLOR, color, -1);
+        gtk_list_store_set(store, &iter, COL_ICON, pix, -1);
         g_free(str);
 		g_object_unref(pix);
     }
@@ -289,7 +296,6 @@ static void clist_refresh(GtkListStore *store, gboolean reload)
 
 static GtkWidget *list;
 static GtkListStore *store;
-static GtkWidget *combo;
 
 typedef struct {
 	GtkWidget *b1;
@@ -309,10 +315,11 @@ typedef struct {
     GtkWidget *m4;
     GtkWidget *m5;
 	GtkWidget *m6;
+	GtkWidget *m8;
 } MI;
 static MI mi;
 
-static void tb_set_states(int s1, int s2, int s3, int s4, int s5, int s6, int s7)
+static void tb_set_states(int s1, int s2, int s3, int s4, int s5, int s6, int s7, int s8)
 {
 	gtk_widget_set_sensitive(tb.b1, s1);
 	gtk_widget_set_sensitive(tb.b2, s2);
@@ -328,13 +335,14 @@ static void tb_set_states(int s1, int s2, int s3, int s4, int s5, int s6, int s7
 	gtk_widget_set_sensitive(mi.m4, s4);
 	gtk_widget_set_sensitive(mi.m5, s5);
 	gtk_widget_set_sensitive(mi.m6, s6);
+	gtk_widget_set_sensitive(mi.m8, s8);
 }
 
 /*
 	Display source code window
 */
 
-extern int update_submenu(GtkWidget*, gpointer);
+extern int update_submenu(GtkWidget*, gpointer);	//dbg_wnds.c
 
 GtkWidget* dbgcode_create_window(void)
 {
@@ -345,13 +353,31 @@ GtkWidget* dbgcode_create_window(void)
 		(tilp_paths_build_glade("dbg_code-2.glade"), "dbgcode_window",
 		 PACKAGE);
 	if (!xml)
-		g_error(_("%s: GUI loading failed !\n"), __FILE__);
+		g_error(_("%s: GUI loading failed!\n"), __FILE__);
 	glade_xml_signal_autoconnect(xml);
 
 	dbox = glade_xml_get_widget(xml, "dbgcode_window");
-#ifdef WND_TRANSIENT
-	gtk_window_set_transient_for(GTK_WINDOW(dbox), GTK_WINDOW(main_wnd));
-#endif
+	if(options3.transient)
+		gtk_window_set_transient_for(GTK_WINDOW(dbox), GTK_WINDOW(main_wnd));
+
+	g_object_set(G_OBJECT(dbox), "allow-shrink", TRUE, NULL);
+
+	/*
+	{
+		GtkStyle *copy;
+
+		GtkStyle *style = gtk_rc_get_style(dbox);
+		printf("=>> %i\n", style->ythickness);
+
+		copy =  gtk_style_copy(style);
+		gtk_style_detach(style);
+
+		style->ythickness = 0;
+
+		gtk_widget_realize(dbox);
+		gtk_style_attach(copy, dbox);		
+	}
+	*/
 
     data = glade_xml_get_widget(xml, "windows1_menu");
     g_signal_connect(G_OBJECT(data), "map", G_CALLBACK(update_submenu), NULL);
@@ -369,7 +395,8 @@ GtkWidget* dbgcode_create_window(void)
     mi.m3 = glade_xml_get_widget(xml, "step_over1");
 	mi.m4 = glade_xml_get_widget(xml, "step_out1");
     mi.m5 = glade_xml_get_widget(xml, "run_to_cursor1");
-    mi.m6 = glade_xml_get_widget(xml, "break1");	
+    mi.m6 = glade_xml_get_widget(xml, "break1");
+    mi.m8 = glade_xml_get_widget(xml, "quit1");
 
 	list = glade_xml_get_widget(xml, "treeview1");
     store = clist_create(list);
@@ -382,6 +409,7 @@ GtkWidget* dbgcode_create_window(void)
 	gtk_widget_hide(data);
 
 	dbgromcall_create_window(xml);
+	dbgromcall_refresh_window();
 
 	return dbox;
 }
@@ -399,14 +427,11 @@ GtkWidget* dbgcode_display_window(void)
 #endif
 
 	gtk_widget_set_sensitive(list, TRUE);	
-	tb_set_states(1, 1, 1, 1, 1, 0, 1);
-    set_other_windows_sensitivity(TRUE);
-     
-    gtk_list_store_clear(store);
-	clist_refresh(store, TRUE);
+	tb_set_states(1, 1, 1, 1, 1, 0, 1, 1);
+	reset_disabled = FALSE;
+	gtk_debugger_enable();
 
-	dbgromcall_refresh_window();
-	gtk_widget_show(dbgw.code);	// always shown
+	gtk_widget_show(dbgw.code);
 
     return dbgw.code;
 }
@@ -437,23 +462,20 @@ void dbgcode_disasm_at(uint32_t addr)
 }
 
 
-GLADE_CB void
-on_quit1_activate                      (GtkMenuItem     *menuitem,
-                                        gpointer         user_data);
-
 GLADE_CB void
 on_run1_activate                       (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
-#if 0
-	tb_set_states(0, 0, 0, 0, 0, 1, 0);
+#ifdef RUN_DBG_OPEN
+	tb_set_states(0, 0, 0, 0, 0, 1, 0, 0);
+	reset_disabled = TRUE;
     gtk_widget_set_sensitive(list, FALSE);
-    set_other_windows_sensitivity(FALSE);
+    gtk_debugger_disable();
 
 	ti68k_debug_step();	// skip possible current bkpt
-    ti68k_engine_start();
+    engine_start();
 #else
-	on_quit1_activate(menuitem, user_data);
+	gtk_debugger_close();
 #endif
 }
 
@@ -462,8 +484,6 @@ GLADE_CB void
 on_step1_activate                      (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
-    static int i =0;
-
 	ti68k_debug_step();
 
 	clist_refresh(store, TRUE);
@@ -471,9 +491,7 @@ on_step1_activate                      (GtkMenuItem     *menuitem,
 	dbgpclog_refresh_window();
     dbgmem_refresh_window();
 	dbgstack_refresh_window();
-
-    // force refresh !
-    while(gtk_events_pending()) gtk_main_iteration_do(FALSE);
+	dbgheap_refresh_window();
 }
 
 
@@ -481,18 +499,18 @@ GLADE_CB void
 on_step_over1_activate                 (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
-	tb_set_states(0, 0, 0, 0, 0, 1, 0);
+	tb_set_states(0, 0, 0, 0, 0, 1, 0, 0);
+	reset_disabled = TRUE;
     ti68k_debug_step_over();
-	tb_set_states(1, 1, 1, 1, 1, 0, 1);
+	tb_set_states(1, 1, 1, 1, 1, 0, 1, 1);
+	reset_disabled = FALSE;
 
 	clist_refresh(store, TRUE);
     dbgregs_refresh_window();
 	dbgpclog_refresh_window();
     dbgmem_refresh_window();
 	dbgstack_refresh_window();
-
-	// force refresh !
-    while(gtk_events_pending()) gtk_main_iteration_do(FALSE);
+	dbgheap_refresh_window();
 }
 
 GLADE_CB void
@@ -506,9 +524,7 @@ on_step_out1_activate                 (GtkMenuItem     *menuitem,
 	dbgpclog_refresh_window();
     dbgmem_refresh_window();
 	dbgstack_refresh_window();
-
-	// force refresh !
-    while(gtk_events_pending()) gtk_main_iteration_do(FALSE);
+	dbgheap_refresh_window();
 }
 
 
@@ -516,7 +532,6 @@ GLADE_CB void
 on_run_to_cursor1_activate             (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
-    //GtkWidget *list = (GtkWidget *)(menuitem);   // arg are swapped, why ?
 	GtkTreeView *view = GTK_TREE_VIEW(list);
 	GtkTreeModel *model = gtk_tree_view_get_model(view);
 	GtkListStore *store = GTK_LIST_STORE(model);
@@ -534,20 +549,23 @@ on_run_to_cursor1_activate             (GtkMenuItem     *menuitem,
     gtk_tree_model_get(model, &iter, COL_ADDR, &str, -1);
     sscanf(str, "%x", &addr);
 
-	tb_set_states(0, 0, 0, 0, 0, 1, 0);
-    set_other_windows_sensitivity(FALSE);
+	tb_set_states(0, 0, 0, 0, 0, 1, 0, 0);
+	reset_disabled = TRUE;
+    gtk_debugger_disable();
 
     ti68k_debug_skip(addr);
     gtk_tree_selection_unselect_iter(selection, &iter);
 
-	tb_set_states(1, 1, 1, 1, 1, 0, 1);
-    set_other_windows_sensitivity(TRUE);
+	tb_set_states(1, 1, 1, 1, 1, 0, 1, 1);
+	reset_disabled = FALSE;
+    gtk_debugger_enable();
     
 	clist_refresh(store, FALSE);
     dbgregs_refresh_window();
 	dbgpclog_refresh_window();
     dbgmem_refresh_window();
 	dbgstack_refresh_window();
+	dbgheap_refresh_window();
 }
 
 
@@ -556,17 +574,17 @@ on_break1_activate                     (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
     // Mode 1 is fastest
-#if 0
-    //GtkWidget *list = (GtkWidget *)(menuitem);   // arg are swapped, why ?
+#ifdef RUN_DBG_OPEN
 	GtkTreeView *view = GTK_TREE_VIEW(list);
 	GtkTreeModel *model = gtk_tree_view_get_model(view);
 	GtkListStore *store = GTK_LIST_STORE(model);
 
-    ti68k_engine_stop();
+    engine_stop();
     gtk_widget_set_sensitive(list, TRUE);
-	tb_set_states(1, 1, 1, 1, 1, 0, 1);
-    set_other_windows_sensitivity(TRUE);
-    clist_refresh(store);
+	tb_set_states(1, 1, 1, 1, 1, 0, 1, 1);
+	reset_disabled = FALSE;
+    gtk_debugger_enable();
+    clist_refresh(store, TRUE);
 #else
     ti68k_debug_break();
 #endif
@@ -578,7 +596,6 @@ GLADE_CB void
 dbgcode_button6_clicked                     (GtkButton       *button,
                                         gpointer         user_data)
 {
-    //GtkWidget *list = GTK_WIDGET(button);   // arg are swapped, why ?
 	GtkTreeView *view = GTK_TREE_VIEW(list);
 	GtkTreeModel *model = gtk_tree_view_get_model(view);
 	GtkListStore *store = GTK_LIST_STORE(model);
@@ -601,11 +618,7 @@ dbgcode_button6_clicked                     (GtkButton       *button,
         ti68k_bkpt_del_address(addr);
 
     clist_refresh(store, FALSE);
-    dbgregs_refresh_window();
-	dbgpclog_refresh_window();
-    dbgmem_refresh_window();
     dbgbkpts_refresh_window();
-	dbgstack_refresh_window();
 }
 
 // Toggle tmp breakpoint
@@ -613,7 +626,6 @@ GLADE_CB void
 dbgcode_button7_clicked                     (GtkButton       *button,
                                         gpointer         user_data)
 {
-    //GtkWidget *list = GTK_WIDGET(button);   // arg are swapped, why ?
 	GtkTreeView *view = GTK_TREE_VIEW(list);
 	GtkTreeModel *model = gtk_tree_view_get_model(view);
 	GtkListStore *store = GTK_LIST_STORE(model);
@@ -636,11 +648,7 @@ dbgcode_button7_clicked                     (GtkButton       *button,
         ti68k_bkpt_del_address(addr | BKPT_TMP_MASK);
 
     clist_refresh(store, FALSE);
-    dbgregs_refresh_window();
-	dbgpclog_refresh_window();
-    dbgmem_refresh_window();
     dbgbkpts_refresh_window();
-	dbgstack_refresh_window();
 }
 
 // Reset cycle counter
@@ -666,7 +674,7 @@ static GtkWidget* display_popup_menu(void)
 	    (tilp_paths_build_glade("dbg_code-2.glade"), "dbgcode_popup",
 	     PACKAGE);
 	if (!xml)
-		g_error(_("%s: GUI loading failed !\n"), __FILE__);
+		g_error(_("%s: GUI loading failed!\n"), __FILE__);
 	glade_xml_signal_autoconnect(xml);
 
 	menu = glade_xml_get_widget(xml, "dbgcode_popup");
@@ -708,7 +716,6 @@ static int export_disasm_to_file(GtkWidget *widget)
 {
 	GtkTreeView *view = GTK_TREE_VIEW(widget);
 	GtkTreeModel *model = gtk_tree_view_get_model(view);
-	GtkListStore *store = GTK_LIST_STORE(model);
     GtkTreeIter iter;
     gchar *str;
     uint32_t addr, start;
@@ -813,15 +820,6 @@ on_treeview1_key_press_event           (GtkWidget       *widget,
     // bind our key
 	switch(event->keyval) 
 	{
-	case GDK_F2:
-		on_set_breakpoint1_activate(NULL, NULL);
-		return FALSE;
-	case GDK_F3:
-		on_set_tmp_bkpt1_activate(NULL, NULL);
-		return FALSE;
-	case GDK_F6:
-		export_disasm_to_file(widget);
-		return FALSE;
 	case GDK_F1:
 		str = g_strdup_printf("%i", row_idx+1);
         path = gtk_tree_path_new_from_string(str);	// restore selection
@@ -829,6 +827,43 @@ on_treeview1_key_press_event           (GtkWidget       *widget,
         gtk_tree_selection_select_path(selection, path);
 		on_set_pc_to_selection1_activate(NULL, user_data);
 		return FALSE;
+
+	case GDK_F2:
+		on_set_breakpoint1_activate(NULL, NULL);
+		return FALSE;
+
+	case GDK_F3:
+		on_set_tmp_bkpt1_activate(NULL, NULL);
+		return FALSE;
+	
+	case GDK_F4:
+		on_run_to_cursor1_activate(NULL, NULL);
+		return FALSE;
+
+	case GDK_F5:
+		on_run1_activate(NULL, NULL);
+		return FALSE;
+
+	case GDK_F6:
+		on_set_pc_to_selection1_activate(NULL, user_data);
+		return FALSE;
+
+	case GDK_F7:
+		on_step1_activate(NULL, NULL);
+		return FALSE;
+
+	case GDK_F8:
+		on_step_over1_activate(NULL, NULL);
+		return FALSE;
+
+	case GDK_F9:
+		on_step_out1_activate(NULL, NULL);
+		return FALSE;
+
+	case GDK_d:
+		export_disasm_to_file(widget);
+		return FALSE;
+
 	case GDK_G:
 	case GDK_g:
 		on_go_to_address1_activate(NULL, user_data);
@@ -950,7 +985,6 @@ on_set_pc_to_selection1_activate       (GtkMenuItem     *menuitem,
     //GtkWidget *list = GTK_WIDGET(user_data);
 	GtkTreeView *view = GTK_TREE_VIEW(list);
 	GtkTreeModel *model = gtk_tree_view_get_model(view);
-	GtkListStore *store = GTK_LIST_STORE(model);
     GtkTreeSelection *selection;
     GtkTreeIter iter;
     gboolean valid;
@@ -966,7 +1000,6 @@ on_set_pc_to_selection1_activate       (GtkMenuItem     *menuitem,
     ti68k_register_set_pc(addr);
     dbgcode_refresh_window();
     dbgregs_refresh_window();
-    dbgmem_refresh_window();
 }
 
 GLADE_CB void
@@ -976,7 +1009,6 @@ on_view_memory1_activate       (GtkMenuItem     *menuitem,
     //GtkWidget *list = GTK_WIDGET(user_data);
 	GtkTreeView *view = GTK_TREE_VIEW(list);
 	GtkTreeModel *model = gtk_tree_view_get_model(view);
-	GtkListStore *store = GTK_LIST_STORE(model);
     GtkTreeSelection *selection;
     GtkTreeIter iter;
     gboolean valid;
@@ -988,12 +1020,9 @@ on_view_memory1_activate       (GtkMenuItem     *menuitem,
 	if(!valid) return;
     gtk_tree_model_get(model, &iter, COL_ADDR, &str, -1);
     sscanf(str, "%x", &addr);
-    
-    printf("addr = %x\n", addr);
     dbgmem_add_tab(addr);
 }
 
-
 GLADE_CB void
 on_treeview1_size_allocate             (GtkWidget       *widget,
                                         GdkRectangle    *allocation,
@@ -1009,10 +1038,12 @@ on_treeview1_size_allocate             (GtkWidget       *widget,
 	gtk_tree_view_get_background_area(view, path, NULL, &rect);
 	g_free(path);
 
-	if(rect.height == 0)
-		return;
+	//printf("allocation: %i %i / rect: %i %i\n", allocation->width, allocation->height, rect.width, rect.height);
 
-	NLINES = allocation->height / rect.height - 1;
+	if(rect.height == 0)
+		NLINES = 1;	// at least 1 line in the window
+	else
+		NLINES = allocation->height / rect.height - 2;
 	//printf("#lines: %i (%i %i)\n", NLINES, allocation->height, rect.height);
 
 	if(old != NLINES)
@@ -1024,3 +1055,188 @@ on_treeview1_size_allocate             (GtkWidget       *widget,
 	old = NLINES;
 #endif
 }
+
+void gdbcallback_disable_debugger(void)
+{
+	if (dbg_on)
+	{
+		tb_set_states(0, 0, 0, 0, 0, 1, 0, 0);
+		gtk_widget_set_sensitive(list, FALSE);
+		gtk_debugger_disable();
+		reset_disabled = TRUE;
+	}
+}
+
+void gdbcallback_enable_debugger(void)
+{
+	if (dbg_on)
+	{
+		gtk_widget_set_sensitive(list, TRUE);
+		tb_set_states(1, 1, 1, 1, 1, 0, 1, 1);
+		gtk_debugger_enable();
+		reset_disabled = FALSE;
+	}
+}
+
+void gdbcallback_refresh_debugger(void)
+{
+	if (dbg_on)
+	{
+		dbgcode_refresh_window();
+		dbgregs_refresh_window();
+		dbgpclog_refresh_window();
+		dbgmem_refresh_window();
+		dbgstack_refresh_window();
+		dbgheap_refresh_window();
+
+		// force refresh !
+		while(gtk_events_pending()) gtk_main_iteration_do(FALSE);
+	}
+}
+
+int dbgcode_quit_enabled(void)
+{
+	return GTK_WIDGET_SENSITIVE(mi.m8);
+}
+
+static int close_debugger_wrapper(gpointer data)
+{
+	gtk_debugger_close();
+	return FALSE;
+}
+
+int gdbcallback_close_debugger(void *clientdata, void *interp, int argc, const char **argv)
+{
+	if (dbg_on && dbgcode_quit_enabled()) g_idle_add(close_debugger_wrapper, NULL);
+	return 0;
+}
+
+GLADE_CB void
+on_linkport1_activate                  (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+	display_loglink_dbox();
+}
+
+// Font change
+
+static GtkWidget *font;
+static gint tmp_type;
+static gchar *tmp_name;
+
+GLADE_CB void
+on_font_activate                       (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+	GladeXML *xml;
+	GtkWidget *dbox;
+	gpointer data;
+	gint result;
+	
+	xml = glade_xml_new
+		(tilp_paths_build_glade("dbg_code-2.glade"), "dbgcode_font", PACKAGE);
+	if (!xml)
+		g_error(_("%s: GUI loading failed!\n"), __FILE__);
+	glade_xml_signal_autoconnect(xml);
+	
+	dbox = glade_xml_get_widget(xml, "dbgcode_font");
+	gtk_dialog_set_alternative_button_order(GTK_DIALOG(dbox), GTK_RESPONSE_OK,
+	                                        GTK_RESPONSE_CANCEL,-1);
+	font = glade_xml_get_widget(xml, "label5");
+
+	tmp_type = options3.dbg_font_type;
+	tmp_name = g_strdup(options3.dbg_font_name);
+
+	if(!tmp_type)
+	{
+		data = glade_xml_get_widget(xml, "radiobutton4");
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(data), TRUE);
+		data = glade_xml_get_widget(xml, "button9");
+		gtk_widget_set_sensitive(GTK_WIDGET(data), FALSE);
+	}
+	else
+	{
+		data = glade_xml_get_widget(xml, "radiobutton6");
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(data), TRUE);
+		if(options3.dbg_font_name)
+			gtk_label_set_text(GTK_LABEL(font), tmp_name);
+		data = glade_xml_get_widget(xml, "button9");
+		gtk_widget_set_sensitive(GTK_WIDGET(data), TRUE);
+	}
+
+	result = gtk_dialog_run(GTK_DIALOG(dbox));
+	switch (result) 
+	{
+	case GTK_RESPONSE_OK:
+		options3.dbg_font_type = tmp_type;
+		if(tmp_name)
+		{
+			g_free(options3.dbg_font_name);
+			options3.dbg_font_name = g_strdup(tmp_name);
+		}
+
+		gtk_debugger_refresh();
+
+		break;
+	default:
+		break;
+	}
+
+	gtk_widget_destroy(dbox);
+	return;
+}
+
+GLADE_CB void
+on_radiobutton4_clicked                (GtkButton       *button,
+                                        gpointer         user_data)
+{
+	PangoContext *context;
+	PangoFontDescription *desc;
+
+	context = gtk_widget_get_pango_context(GTK_WIDGET(button));
+	desc = pango_context_get_font_description(context);
+	gtk_label_set_text(GTK_LABEL(font), pango_font_description_to_string(desc));
+
+	gtk_widget_set_sensitive(GTK_WIDGET(button), FALSE);
+	tmp_type = 0;
+}
+
+GLADE_CB void
+on_radiobutton6_clicked                (GtkButton       *button,
+                                        gpointer         user_data)
+{
+	if(tmp_name)
+		gtk_label_set_text(GTK_LABEL(font), tmp_name);
+
+	gtk_widget_set_sensitive(GTK_WIDGET(button), TRUE);
+	tmp_type = 1;
+}
+
+GLADE_CB void
+on_dbgcode_button9_clicked             (GtkButton       *button,
+                                        gpointer         user_data)
+{
+	gint result;
+	GtkWidget *dlg;
+	gchar *str;
+	
+	dlg = gtk_font_selection_dialog_new(_("Choose font..."));
+	if(tmp_name)
+		gtk_font_selection_dialog_set_font_name(GTK_FONT_SELECTION_DIALOG(dlg), tmp_name);
+	result = gtk_dialog_run(GTK_DIALOG(dlg));	
+
+	switch (result) 
+	{ 
+	case GTK_RESPONSE_OK:
+		str = gtk_font_selection_dialog_get_font_name(GTK_FONT_SELECTION_DIALOG(dlg));
+		g_free(tmp_name);
+		tmp_name = g_strdup(str);
+		gtk_label_set_text(GTK_LABEL(font), str);
+		g_free(str);
+		break;
+	default:
+		break;
+	}
+
+	gtk_widget_destroy(dlg);
+}
