@@ -1,13 +1,14 @@
 /* Hey EMACS -*- linux-c -*- */
-/* $Id: dbg_mem.c 1673 2005-08-22 15:27:17Z roms $ */
+/* $Id: dbg_mem.c 2753 2007-12-30 23:14:15Z kevinkofler $ */
 
-/*  TiEmu - an TI emulator
+/*  TiEmu - Tiemu Is an EMUlator
  *
  *  Copyright (c) 2000-2001, Thomas Corvazier, Romain Lievin
  *  Copyright (c) 2001-2003, Romain Lievin
  *  Copyright (c) 2003, Julien Blache
  *  Copyright (c) 2004, Romain Livin
  *  Copyright (c) 2005, Romain Livin
+ *  Copyright (c) 2006-2007, Kevin Kofler
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -42,7 +43,8 @@
 #include "support.h"
 #include "ti68k_int.h"
 #include "struct.h"
-#include "dbg_all.h"
+#include "dbg_wnds.h"
+#include "mem_map.h"
 
 #define FORCE_REFRESH
 #define DUMP_SIZE       128
@@ -68,8 +70,6 @@ enum {
 #define CLIST_NVCOLS	(18)
 #define CLIST_NCOLS		(18 + 20)
 
-#define FONT_NAME	"courier"
-
 static gint column2index(GtkWidget *list, GtkTreeViewColumn * column)
 {
 	gint i;
@@ -135,22 +135,21 @@ static void renderer_edited(GtkCellRendererText * cell,
     for(i = 0; (i < (int)strlen(str_data)/2) && ((col+i) <= COL_F); i++)
     {
         char digits[3];
+		int offset = col - COL_0 + i;
 
         strncpy(digits, &new_text[2*i], 2); 
         digits[2] = '\0';
 
         sscanf(str_addr, "%x", &addr);
         sscanf(digits, "%x", &data);
-		mem_wr_byte(addr, (uint8_t)data);
+		mem_wr_byte(addr + offset, (uint8_t)data);
 
 		// don't rely on typed value
-		data = mem_rd_byte(addr);
+		data = mem_rd_byte(addr + offset);
 		sprintf(digits, "%02x", data);
-		ascii[col - COL_0 + i] = (isprint(data) && !iscntrl(data) ? data : '.');
+		ascii[offset] = (isprint(data) && !iscntrl(data) ? data : '.');
 
 		gtk_list_store_set(store, &iter, col+i, digits, -1);
-		addr += (col - COL_0) + i;
-
 		dbgstack_refresh_window();	// refresh stack, too
     }
 
@@ -216,6 +215,7 @@ static GtkWidget* clist_create(GtkListStore **st)
   
 	i = COL_ADDR;
 	renderer = gtk_cell_renderer_text_new();
+	set_renderer_pad(renderer);
 	gtk_tree_view_insert_column_with_attributes(view, -1, 
             text[i], renderer, 
             "text", i,
@@ -226,7 +226,7 @@ static GtkWidget* clist_create(GtkListStore **st)
     for (i = COL_0; i <= COL_F; i++)
     {
     	renderer = gtk_cell_renderer_text_new();
-
+		set_renderer_pad(renderer);
         gtk_tree_view_insert_column_with_attributes(view, -1, 
             text[i], renderer, 
             "text", i, 
@@ -242,6 +242,7 @@ static GtkWidget* clist_create(GtkListStore **st)
 
 	i = COL_ASCII;
 	renderer = gtk_cell_renderer_text_new();
+	set_renderer_pad(renderer);
 	gtk_tree_view_insert_column_with_attributes(view, -1, 
             text[i], renderer, 
             "text", i,
@@ -314,10 +315,14 @@ static void clist_populate(GtkListStore *store, uint32_t start, int length)
 		gtk_list_store_set(store, &iter, 
 			COL_ADDR, str, 
 			COL_GRAY, &gray,
-			COL_FONT, FONT_NAME,            
 			-1);
 		g_free(str);
 
+		if(options3.dbg_font_type)
+			gtk_list_store_set(store, &iter, COL_FONT, options3.dbg_font_name, -1);
+		else
+			gtk_list_store_set(store, &iter, COL_FONT, "courier", -1);
+
 		for(i = COL_0; i <= COL_F; i++)
 		{
 			if(addr + (i-COL_0) != 0x60000f)
@@ -407,9 +412,8 @@ GtkWidget* dbgmem_create_window(void)
 	glade_xml_signal_autoconnect(xml);
 	
 	dbox = glade_xml_get_widget(xml, "dbgmem_window");
-#ifdef WND_TRANSIENT
-	gtk_window_set_transient_for(GTK_WINDOW(dbox), GTK_WINDOW(main_wnd));
-#endif
+	if(options3.transient)
+		gtk_window_set_transient_for(GTK_WINDOW(dbox), GTK_WINDOW(main_wnd));
 
     notebook = glade_xml_get_widget(xml, "notebook1");
     gtk_notebook_popup_enable(GTK_NOTEBOOK(notebook));
@@ -431,8 +435,6 @@ GtkWidget* dbgmem_display_window(void)
 		gtk_window_iconify(GTK_WINDOW(dbgw.mem));
 #endif
 
-	refresh_page(0, 0);
-
 	if(!GTK_WIDGET_VISIBLE(dbgw.mem) && !options3.mem.closed)
 		gtk_widget_show(dbgw.mem);
 
@@ -515,6 +517,80 @@ dbgmem_button5_clicked                     (GtkButton       *button,
 	refresh_page(page, 0);
 }
 
+/***** Mem Map popup menu *****/
+
+static void memmap_menu_activate(GtkMenuItem* menuitem, gpointer user_data)
+{
+	guint32 addr = GPOINTER_TO_INT(user_data);
+	gchar *str;
+	
+	str = g_strdup_printf("%06x", addr);
+	notebook_add_page(notebook, str);
+	g_free(str);
+}
+
+static GtkWidget* memmap_menu(void)
+{
+	GtkWidget *menu;
+	GtkWidget *item;
+	int result;
+	MEM_MAP **ptr, **array = NULL;
+
+
+	menu = gtk_menu_new();
+	g_object_set_data_full(G_OBJECT(menu), "memmap_menu",
+			       gtk_widget_ref(menu),
+			       (GDestroyNotify)g_object_unref);
+
+	// (re)load mem map
+	result = ti68k_debug_load_memmap(inst_paths.misc_dir);
+	if(result == -1)
+	{
+		return menu;
+	}
+
+	array = memmap_array();
+	if(array == NULL)
+		return menu;
+	
+	for(ptr = array; *ptr; ptr++)
+	{
+		MEM_MAP *s = *ptr;
+		char *label;
+
+		label = g_strdup_printf("%06x-%06x : %s", s->addr, s->addr + s->size - 1, s->name);
+
+		item = gtk_menu_item_new_with_label(label);
+		g_object_set_data_full(G_OBJECT(menu), "c_drive",
+					   gtk_widget_ref(item),
+					   (GDestroyNotify)g_object_unref);
+		gtk_widget_show(item);
+
+		gtk_container_add(GTK_CONTAINER(menu), item);
+		g_signal_connect((gpointer)item, "activate",
+					   G_CALLBACK(memmap_menu_activate),
+					   GINT_TO_POINTER(s->addr));
+
+		g_free(label);
+	}
+
+	return menu;
+}
+
+GLADE_CB void
+dbgmem_button6_clicked                     (GtkButton       *button,
+                                        gpointer         user_data)
+{
+	GtkWidget *menu;
+	guint butt = 0;
+	guint32 time;
+
+	time = gtk_get_current_event_time();
+	menu = memmap_menu();
+	gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, butt, time);
+	gtk_widget_show(menu);
+}
+
 GLADE_CB void
 on_notebook1_switch_page               (GtkNotebook     *notebook,
                                         GtkNotebookPage *page,
@@ -524,6 +600,8 @@ on_notebook1_switch_page               (GtkNotebook     *notebook,
 	refresh_page(page_num, 0);
 }
 
+/***** Misc *****/
+
 static void refresh_page(int page, int offset)
 {
 	GtkNotebook *nb = GTK_NOTEBOOK(notebook);
@@ -619,7 +697,7 @@ static GtkWidget* display_popup_menu(void)
 	    (tilp_paths_build_glade("dbg_mem-2.glade"), "dbgmem_popup",
 	     PACKAGE);
 	if (!xml2)
-		g_error(_("%s: GUI loading failed !\n"), __FILE__);
+		g_error(_("%s: GUI loading failed!\n"), __FILE__);
 	glade_xml_signal_autoconnect(xml2);
 
 	menu = glade_xml_get_widget(xml2, "dbgmem_popup");
@@ -722,7 +800,7 @@ on_go_to_address2_activate             (GtkMenuItem     *menuitem,
 	g_free(str);
 }
 
-static uint32_t on_disassemble_common  (GtkMenuItem     *menuitem,
+static uint32_t column2address         (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
 	GtkNotebook *nb = GTK_NOTEBOOK(notebook);
@@ -770,7 +848,7 @@ GLADE_CB void
 on_dissassemble1_activate              (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
-    uint32_t addr = on_disassemble_common(menuitem, user_data);
+    uint32_t addr = column2address(menuitem, user_data);
     
     // populate code at this address
     dbgcode_disasm_at(addr);
@@ -780,14 +858,67 @@ GLADE_CB void
 on_disassemble_indirect1_activate              (GtkMenuItem     *menuitem,
                                                 gpointer         user_data)
 {
-	uint32_t addr = on_disassemble_common(menuitem, user_data);
+	uint32_t addr = column2address(menuitem, user_data);
 
 	// populate code at the address contained at this address
 	dbgcode_disasm_at(mem_rd_long(addr));
 }
 
+GLADE_CB void
+on_dbgmem_view_memory1_activate                (GtkMenuItem     *menuitem,
+                                                gpointer         user_data)
+{
+	uint32_t addr = column2address(menuitem, user_data);
+	gchar *str;
+
+	// open a new tab at the address contained at this address	
+	str = g_strdup_printf("%06x", mem_rd_long(addr));
+	notebook_add_page(notebook, str);
+	g_free(str);
+}
+
 static void search_next(void);
 
+GLADE_CB gboolean
+on_dbgmem_key_press_event              (GtkWidget       *widget,
+                                        GdkEventKey     *event,
+                                        gpointer         user_data)
+{
+	GtkNotebook *nb = GTK_NOTEBOOK(notebook);
+
+    switch(event->keyval) 
+	{
+	case GDK_F1:
+		dbgmem_button1_clicked(NULL, NULL);
+		return TRUE;
+	case GDK_F2:
+		dbgmem_button2_clicked(NULL, NULL);
+		return TRUE;
+	case GDK_F3:
+		dbgmem_button3_clicked(NULL, NULL);
+		return TRUE;
+	case GDK_F4:
+		dbgmem_button4_clicked(NULL, NULL);
+		return TRUE;
+	case GDK_F5:
+		dbgmem_button5_clicked(NULL, NULL);
+		return TRUE;
+	case GDK_F6:
+		dbgmem_button6_clicked(NULL, NULL);
+		return TRUE;
+	case GDK_F7:
+		gtk_notebook_next_page(nb);
+		return TRUE;
+	case GDK_F8:
+		gtk_notebook_prev_page(nb);
+		return TRUE;
+	default:
+		return FALSE;
+	}
+
+    return FALSE;
+}
+
 static gboolean
 on_treeview_key_press_event            (GtkWidget       *widget,
                                         GdkEventKey     *event,
@@ -795,7 +926,6 @@ on_treeview_key_press_event            (GtkWidget       *widget,
 {
     GtkTreeView *view = GTK_TREE_VIEW(widget);
 	GtkTreeModel *model = gtk_tree_view_get_model(view);
-	GtkListStore *store = GTK_LIST_STORE(model);
     GtkTreeIter iter;
     GtkTreePath *path;
     GtkTreeViewColumn *column;
@@ -892,7 +1022,7 @@ on_treeview_key_press_event            (GtkWidget       *widget,
 
         return FALSE;
 
-    case GDK_F3:
+    case GDK_n:
 		search_next();
 		return TRUE;
 
@@ -912,6 +1042,14 @@ on_treeview_key_press_event            (GtkWidget       *widget,
             return TRUE;
         }
 
+	case GDK_plus:
+		dbgmem_button1_clicked(NULL, NULL);
+		return FALSE;
+
+	case GDK_minus:
+		dbgmem_button2_clicked(NULL, NULL);
+		return FALSE;
+
 	default:
 		return FALSE;
 	}
@@ -934,7 +1072,7 @@ gint display_dbgmem_address(uint32_t *addr)
 	xml = glade_xml_new
 		(tilp_paths_build_glade("dbg_mem-2.glade"), "dbgmem_address", PACKAGE);
 	if (!xml)
-		g_error(_("%s: GUI loading failed !\n"), __FILE__);
+		g_error(_("%s: GUI loading failed!\n"), __FILE__);
 	glade_xml_signal_autoconnect(xml);
 	
 	entry = glade_xml_get_widget(xml, "entry1");
@@ -945,6 +1083,8 @@ gint display_dbgmem_address(uint32_t *addr)
 	gtk_editable_select_region(GTK_EDITABLE(entry), 0, -1);
 	
 	dbox = glade_xml_get_widget(xml, "dbgmem_address");	
+	gtk_dialog_set_alternative_button_order(GTK_DIALOG(dbox), GTK_RESPONSE_OK,
+	                                        GTK_RESPONSE_CANCEL,-1);
 	result = gtk_dialog_run(GTK_DIALOG(dbox));
 	
 	switch (result) 
@@ -977,8 +1117,8 @@ static gint search_engine(char *str, int ascii, int casse, uint32_t *address, in
     gint i;
     uint32_t addr;
 
-    data = g_strdup(str);
-    len = strlen(data);
+    data = (uint8_t *) g_strdup((gchar *) str);
+    len = strlen((char *) data);
 
     // ASCII mode ?
     if(!ascii)
@@ -1003,10 +1143,12 @@ static gint search_engine(char *str, int ascii, int casse, uint32_t *address, in
         for(i = 0; i < len; i++)
         {
             char digits[3];
+            unsigned int temp;
 
             strncpy(digits, &tmp[2*i], 2); 
             digits[2] = '\0';
-            sscanf(digits, "%02x", &data[i]);
+            sscanf(digits, "%02x", &temp);
+            data[i] = temp;
         }
 
         g_free(tmp);
@@ -1058,7 +1200,6 @@ static gint search_highlight(uint32_t blk_beg, uint32_t blk_end, int state)
 
     GdkColor white, green;
 	gboolean success;
-    GdkColor *color = &green;
 
 	gdk_color_parse("White", &white);
 	gdk_colormap_alloc_colors(gdk_colormap_get_system(), &white, 1,
@@ -1155,7 +1296,7 @@ gint display_dbgmem_search(uint32_t addr)
 	xml = glade_xml_new
 		(tilp_paths_build_glade("dbg_mem-2.glade"), "dbgmem_search", PACKAGE);
 	if (!xml)
-		g_error(_("%s: GUI loading failed !\n"), __FILE__);
+		g_error(_("%s: GUI loading failed!\n"), __FILE__);
 	glade_xml_signal_autoconnect(xml);
 	
     if(old_str == NULL)
@@ -1168,7 +1309,10 @@ gint display_dbgmem_search(uint32_t addr)
     check2 = glade_xml_get_widget(xml, "checkbutton2");
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check2), TRUE);
 
-	dbox = glade_xml_get_widget(xml, "dbgmem_search");	
+	dbox = glade_xml_get_widget(xml, "dbgmem_search");
+	/* Find, Find Next, Cancel */
+	gtk_dialog_set_alternative_button_order(GTK_DIALOG(dbox), GTK_RESPONSE_OK,
+	                                        GTK_RESPONSE_APPLY, GTK_RESPONSE_CANCEL,-1);
 	
     
     for(result = 0;;)
