@@ -1,13 +1,13 @@
 /* Hey EMACS -*- linux-c -*- */
-/* $Id: rcfile.c 1637 2005-08-05 17:34:58Z roms $ */
+/* $Id: rcfile.c 2730 2007-12-16 15:54:33Z roms $ */
 
-/*  TiEmu - an TI emulator
+/*  TiEmu - Tiemu Is an EMUlator
  *
  *  Copyright (c) 2000-2001, Thomas Corvazier, Romain Lievin
  *  Copyright (c) 2001-2003, Romain Lievin
  *  Copyright (c) 2003, Julien Blache
  *  Copyright (c) 2004, Romain Livin
- *  Copyright (c) 2005, Romain Livin, Kevin Kofler
+ *  Copyright (c) 2005-2007, Romain Livin, Kevin Kofler
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -49,20 +49,22 @@
 #include "files.h"
 #include "ti68k_int.h"
 #include "screenshot.h"
-#include "printl.h"
+#include "logging.h"
 
 static char *rc_file; 
 
 #define MAXCHARS 256
 
+#define DEFAULT_FONT_NAME	"Courier"
+
 /*
   Return the path where the config file is stored
 */
 int rcfile_get_path(char **path)
 {
 	*path = g_strconcat(inst_paths.home_dir, 
-			    DIR_SEPARATOR_S, CONF_DIR, 
-			    DIR_SEPARATOR, INI_FILE, NULL); 
+			    G_DIR_SEPARATOR_S, CONF_DIR, 
+			    G_DIR_SEPARATOR_S, INI_FILE, NULL); 
 	return 0;
 }
 
@@ -71,7 +73,7 @@ int rcfile_get_path(char **path)
 */
 static void stop (int line)
 {
-	printl(2, _("Configuration file error at line %i.\n"), line);
+	tiemu_warning(_("Configuration file error at line %i."), line);
 }
 
 /*
@@ -90,17 +92,25 @@ static char *find_str(char *s, const char *t)
 /* Chech whether a RC file exists */
 int rcfile_exist(void)
 {
+	int result;
+
 	rcfile_get_path(&rc_file);
   
-	return !access(rc_file, F_OK);
+	result = !access(rc_file, F_OK);
+	g_free(rc_file);
+	return result;
 }
 
 /* Delete the RC file */
 int rcfile_delete(void)
 {
+	int result;
+
 	rcfile_get_path(&rc_file);
 
-	return unlink(rc_file);
+	result = unlink(rc_file);
+	g_free(rc_file);
+	return result;
 }
 
 /* Return TiLP version number */
@@ -119,15 +129,23 @@ int rcfile_get_version(char *version)
 	rcfile_get_path(&rc_file);
 
 	txt = fopen(rc_file, "rt");
+	g_free(rc_file);
 	if (txt == NULL)
 		return -1;
 
 	for (i = 0; i < 5; i++)
-		fgets(buffer, 256, txt);
+		if (!fgets(buffer, 256, txt))
+		{
+			fclose(txt);
+			return -1;
+		}
 
 	p = strchr(buffer, '=');
 	if (p == NULL)
+	{
+		fclose(txt);
 		return -1;
+	}
 
 	strcpy(version, ++p);
 
@@ -139,6 +157,9 @@ int rcfile_get_version(char *version)
 	if (p)
 		*p = '\0';
 
+	if (fclose(txt))
+		return -1;
+
 	return 0;
 }
 
@@ -155,14 +176,15 @@ void rcfile_read(void)
 	rcfile_get_path(&rc_file);
 
 	txt=fopen(rc_file, "rt");
+	g_free(rc_file);
 	if(txt == NULL) {
-		printl(0, _("Configuration file not found, use default values. You can create one by the 'File|Save config' command menu.\n"));
-		//gif->msg_box(_("Information"), _("Configuration file not found, use default values. You can create one by the 'File|Save config' command menu.\n"));
+		tiemu_info(_("Configuration file not found, use default values. You can create one by the 'File|Save config' command menu."));
 		return;
 	}
 
 	while(!feof(txt)) {
-		fgets(buffer, 256, txt);
+		if (!fgets(buffer, 256, txt))
+			break;
 		l++;
 		buffer[strlen(buffer)-1]='\0';
 
@@ -174,163 +196,40 @@ void rcfile_read(void)
 	if(buffer[0]=='#' || !strlen(buffer)) continue;
       
 	/* Common part with TiLP: hardware section */
-	if ((p = find_str(buffer, "calctype="))) {
-		if (!strcmp(p, "v200"))
-			link_cable.calc_type = CALC_V200;
-
-		else if (!strcmp(p, "ti92+"))
-			link_cable.calc_type = CALC_TI92P;
-
-		else if (!strcmp(p, "ti92"))
-			link_cable.calc_type = CALC_TI92;
-
-		else if (!strcmp(p, "ti89"))
-			link_cable.calc_type = CALC_TI89;
-
-		else if (!strcmp(p, "ti86"))
-			link_cable.calc_type = CALC_TI86;
-
-		else if (!strcmp(p, "ti85"))
-			link_cable.calc_type = CALC_TI85;
-
-		else if (!strcmp(p, "ti83+"))
-			link_cable.calc_type = CALC_TI83P;
-
-		else if (!strcmp(p, "ti83"))
-			link_cable.calc_type = CALC_TI83;
-
-		else if (!strcmp(p, "ti82"))
-			link_cable.calc_type = CALC_TI82;
-
-		else if (!strcmp(p, "ti73"))
-			link_cable.calc_type = CALC_TI73;
-
-		else
-			stop(l);
+	if ((p = find_str(buffer, "cable_model="))) 
+	{
+		linkp.cable_model = ticables_string_to_model(p);
 		continue;
 	}
 	
-	if ((p = find_str(buffer, "linktype="))) {
-		if (!strcmp(p, "parallel"))
-			link_cable.link_type = LINK_PAR;
-
-		else if (!strcmp(p, "serial"))
-			link_cable.link_type = LINK_SER;
-
-		else if (!strcmp(p, "TIGraphLink"))
-			link_cable.link_type = LINK_TGL;
-
-		else if (!strcmp(p, "VTi"))
-			link_cable.link_type = LINK_VTI;
-
-		else if (!strcmp(p, "TiEmulator"))
-			link_cable.link_type = LINK_TIE;
-
-		else if (!strcmp(p, "virtual"))
-			link_cable.link_type = LINK_VTL;
-
-		else if (!strcmp(p, "UsbGraphLink"))
-			link_cable.link_type = LINK_UGL;
-
-		else if (!strcmp(p, "none"))
-			link_cable.link_type = LINK_NUL;
-
-		else
-			stop(l);
+	if ((p = find_str(buffer, "cable_port="))) 
+	{
+		linkp.cable_port = ticables_string_to_port(p);
 		continue;
 	}
 	
-	if ((p = find_str(buffer, "adr_port="))) {
-		sscanf(p, "0x%03X", &(link_cable.io_addr));
-
-		//check_access();
+	if ((p = find_str(buffer, "cable_timeout="))) 
+	{
+		sscanf(p, "%i", &(linkp.cable_timeout));
 		continue;
 	}
 	
-	if ((p = find_str(buffer, "device="))) {
-		strcpy(link_cable.device, p);
-	}
-	
-	if ((p = find_str(buffer, "timeout="))) {
-		sscanf(p, "%i", &(link_cable.timeout));
+	if ((p = find_str(buffer, "cable_delay="))) {
+		sscanf(p, "%i", &(linkp.cable_delay));
 		continue;
 	}
-	
-	if ((p = find_str(buffer, "delay="))) {
-		sscanf(p, "%i", &(link_cable.delay));
-		continue;
+
+	if( (p=find_str(buffer, "qs_file=")) ) 
+	{
+	  g_free(options.qs_file);
+	  options.qs_file = g_strdup(p);
+	  continue;
 	}
-	
-	if ((p = find_str(buffer, "port="))) {
-		if (!strcmp(p, "user"))
-			link_cable.port = USER_PORT;
 
-		else if (!strcmp(p, "parallel port #1"))
-			link_cable.port = PARALLEL_PORT_1;
-
-		else if (!strcmp(p, "parallel port #2"))
-			link_cable.port = PARALLEL_PORT_2;
-
-		else if (!strcmp(p, "parallel port #3"))
-			link_cable.port = PARALLEL_PORT_3;
-
-		else if (!strcmp(p, "serial port #1"))
-			link_cable.port = SERIAL_PORT_1;
-
-		else if (!strcmp(p, "serial port #2"))
-			link_cable.port = SERIAL_PORT_2;
-
-		else if (!strcmp(p, "serial port #3"))
-			link_cable.port = SERIAL_PORT_3;
-
-		else if (!strcmp(p, "serial port #4"))
-			link_cable.port = SERIAL_PORT_4;
-
-		else if (!strcmp(p, "virtual port #1"))
-			link_cable.port = VIRTUAL_PORT_1;
-
-		else if (!strcmp(p, "virtual port #2"))
-			link_cable.port = VIRTUAL_PORT_2;
-
-		else if (!strcmp(p, "USB port #1"))
-			link_cable.port = USB_PORT_1;
-
-		else if (!strcmp(p, "USB port #2"))
-			link_cable.port = USB_PORT_2;
-
-		else if (!strcmp(p, "USB port #3"))
-			link_cable.port = USB_PORT_3;
-
-		else if (!strcmp(p, "USB port #4"))
-			link_cable.port = USB_PORT_4;
-
-		else if (!strcmp(p, "null"))
-			link_cable.port = NULL_PORT;
-
-		else
-			stop(l);
-		continue;
-	}
-	
-	if ((p = find_str(buffer, "method="))) {
-		if (!strcmp(p, "automatic"))
-			link_cable.method = IOM_AUTO;
-
-		else if (!strcmp(p, "asm"))
-			link_cable.method = IOM_ASM;
-
-		else if (!strcmp(p, "api"))
-			link_cable.method = IOM_API;
-
-		else if (!strcmp(p, "driver"))
-			link_cable.method = IOM_DRV;
-
-		else if (!strcmp(p, "null"))
-			link_cable.method = IOM_NULL;
-
-		else
-			stop(l);
-		continue;
+	if( (p=find_str(buffer, "qs_enabled=")) ) 
+	{
+	  sscanf(p, "%i", &(options.qs_enabled));
+	  continue;
 	}
 
       /* GtkTiEmu specific part: emulator section */
@@ -362,12 +261,6 @@ void rcfile_read(void)
 	  continue;
 	}
 
-	if( (p=find_str(buffer, "background=")) ) 
-	{
-		// for compat
-	  sscanf(p, "%i", &(options.skin));
-	  continue;
-	}
 	if( (p=find_str(buffer, "skin=")) ) 
 	{
 	  sscanf(p, "%i", &(options.skin));
@@ -384,8 +277,8 @@ void rcfile_read(void)
 		  char *q;
 		  options.view = VIEW_CUSTOM;
 		  p=find_str(buffer, "view_mode=custom");
-		  q = strchr(p, ',');
-		  if(q) *q = '.';
+		  /*q = strchr(p, ',');
+		  if(q) *q = '.';*/
 		  sscanf(p, " (%f)", &(options.scale));
 		  if(options.scale < 0.01)
 			  options.scale = 1.0;
@@ -425,6 +318,7 @@ void rcfile_read(void)
 	  else if(!strcmp(p, "ico")) options2.format=IMG_ICO;
 	  else if(!strcmp(p, "eps")) options2.format=IMG_EPS;
 	  else if(!strcmp(p, "pdf")) options2.format=IMG_PDF;
+	  else if(!strcmp(p, "bmp")) options2.format=IMG_BMP;
 	  else stop(l);
 	  continue;
 	}
@@ -444,6 +338,13 @@ void rcfile_read(void)
 	  else stop(l);
 	  continue;
 	}
+
+	if( (p=find_str(buffer, "screen_folder=")) )
+	{
+	  g_free(options2.folder);
+	  options2.folder = g_strdup(p);
+	  continue;
+	}
       
 	if( (p=find_str(buffer, "screen_file=")) )
 	{
@@ -457,6 +358,26 @@ void rcfile_read(void)
 	  sscanf(p, "%i", &(options2.counter));
 	  continue;
 	}
+
+	if( (p=find_str(buffer, "screen_shots=")) )
+	{
+	  sscanf(p, "%i", &(options2.shots));
+	  continue;
+	}
+
+	if( (p=find_str(buffer, "screen_skips=")) )
+	{
+	  sscanf(p, "%i", &(options2.skips));
+	  continue;
+	}
+
+	if( (p=find_str(buffer, "clipboard=")) )
+	{
+	  if(!strcmp(p, "no")) options2.clipboard = 0;
+	  else if(!strcmp(p, "yes")) options2.clipboard = 1;
+
+	  continue;
+	}
 	
 	if( (p=find_str(buffer, "console=")) )
 	{
@@ -477,7 +398,7 @@ void rcfile_read(void)
 	{
 	  if(!strcmp(p, "old")) options.fs_type = 0;
 	  else if(!strcmp(p, "new")) options.fs_type = 1;
-	  else if(!strcmp(p, "win32")) options.fs_type = 2;
+	  else if(!strcmp(p, "native") || !strcmp(p, "win32")) options.fs_type = 2;
 	  else if(!strcmp(p, "kde")) options.fs_type = 3;
 	}
 	
@@ -570,6 +491,44 @@ void rcfile_read(void)
 		  &(options3.iop.minimized), &(options3.iop.closed));
 	  continue;
 	}
+	if( (p=find_str(buffer, "dock_wnd=")) )
+	{
+	  sscanf(p, "(%i;%i;%i;%i;%i;%i)", 
+		  &(options3.dock.rect.x), &(options3.dock.rect.y), 
+		  &(options3.dock.rect.w), &(options3.dock.rect.h),
+		  &(options3.dock.minimized), &(options3.dock.closed));
+	  continue;
+	}
+	if( (p=find_str(buffer, "wnd_transient=")) )
+	{
+	  if(!strcmp(p, "no")) options3.transient = 0;
+	  else if(!strcmp(p, "yes")) options3.transient = 1;
+	  else stop(l);
+	  continue;
+	}
+
+	if( (p=find_str(buffer, "dbg_font_type=")) )
+	{
+	  if(!strcmp(p, "default")) options3.dbg_font_type = 0;
+	  else if(!strcmp(p, "custom")) options3.dbg_font_type = 1;
+	  else stop(l);
+	  continue;
+	}
+	if( (p=find_str(buffer, "dbg_font_name=")) )
+	{
+		g_free(options3.dbg_font_name);
+		options3.dbg_font_name = g_strdup(p);
+		continue;
+	}
+
+	if( (p=find_str(buffer, "dbg_dock=")) )
+	{
+	  if(!strcmp(p, "no")) options3.dbg_dock = 0;
+	  else if(!strcmp(p, "yes")) options3.dbg_dock = 1;
+	  else stop(l);
+	  continue;
+	}
+
     }
   fclose(txt);
 
@@ -591,17 +550,18 @@ void rcfile_write(void)
 	_mkdir(CONF_DIR);
 #endif
 	txt=fopen(rc_file, "wt");
+	g_free(rc_file);
 
 	if(txt==NULL) {
 		//gif->msg_box(_("Error"), _("Unable to write the config file (~/.tilp or tilp.ini).\n"));
 		return;
 	}
 
-	fprintf(txt, "# Config file for TiEmu II\n");
-	fprintf(txt, "  (C) Romain Lievin & Thomas Corvazier  2000-2001\n");
-	fprintf(txt, "  (C) Romain Lievin 2001-2003\n");
-	fprintf(txt, "  (C) Julien Blache 2003\n");
-	fprintf(txt, "  (C) Romain Livin 2004-2005\n");
+	fprintf(txt, "# Config file for TiEmu 3\n");
+	fprintf(txt, "# (C) Romain Lievin & Thomas Corvazier  2000-2001\n");
+	fprintf(txt, "# (C) Romain Lievin 2001-2003\n");
+	fprintf(txt, "# (C) Julien Blache 2003\n");
+	fprintf(txt, "# (C) Romain Livin 2004-2005\n");
 	fprintf(txt, "# Warning: any comments that you add to this file WILL be overwritten\n");
 	fprintf(txt, "\n");
 
@@ -614,156 +574,25 @@ void rcfile_write(void)
 	fprintf(txt, "#\n");
 	fprintf(txt, "\n");
 
-	fprintf(txt, "# Calculator type\n");
-	fprintf(txt, "calctype=");
-	switch (link_cable.calc_type) {
-	case CALC_V200:
-		fprintf(txt, "v200\n");
-		break;
-	case CALC_TI92P:
-		fprintf(txt, "ti92+\n");
-		break;
-	case CALC_TI92:
-		fprintf(txt, "ti92\n");
-		break;
-	case CALC_TI89:
-		fprintf(txt, "ti89\n");
-		break;
-	case CALC_TI86:
-		fprintf(txt, "ti86\n");
-		break;
-	case CALC_TI85:
-		fprintf(txt, "ti85\n");
-		break;
-	case CALC_TI83P:
-		fprintf(txt, "ti83+\n");
-		break;
-	case CALC_TI83:
-		fprintf(txt, "ti83\n");
-		break;
-	case CALC_TI82:
-		fprintf(txt, "ti82\n");
-		break;
-	case CALC_TI73:
-		fprintf(txt, "ti73\n");
-		break;
-	}
-	fprintf(txt, "\n");
-
 	fprintf(txt, "# Link cable type\n");
-	fprintf(txt, "linktype=");
-	switch (link_cable.link_type) {
-	case LINK_PAR:
-		fprintf(txt, "parallel\n");
-		break;
-	case LINK_SER:
-		fprintf(txt, "serial\n");
-		break;
-	case LINK_TGL:
-		fprintf(txt, "TIGraphLink\n");
-		break;
-	case LINK_VTL:
-		fprintf(txt, "virtual\n");
-		break;
-	case LINK_TIE:
-		fprintf(txt, "TiEmulator\n");
-		break;
-	case LINK_VTI:
-		fprintf(txt, "VTi\n");
-		break;
-	case LINK_UGL:
-		fprintf(txt, "UsbGraphLink\n");
-		break;
-	default:
-		fprintf(txt, "none\n");
-		break;
-	}
+	fprintf(txt, "cable_model=%s\n", ticables_model_to_string(linkp.cable_model));
 	fprintf(txt, "\n");
 
 	fprintf(txt, "# Port to use (serial, parallel, ...\n");
-	fprintf(txt, "port=");
-	switch (link_cable.port) {
-	case USER_PORT:
-		fprintf(txt, "user\n");
-		break;
-	case PARALLEL_PORT_1:
-		fprintf(txt, "parallel port #1\n");
-		break;
-	case PARALLEL_PORT_2:
-		fprintf(txt, "parallel port #2\n");
-		break;
-	case PARALLEL_PORT_3:
-		fprintf(txt, "parallel port #3\n");
-		break;
-	case SERIAL_PORT_1:
-		fprintf(txt, "serial port #1\n");
-		break;
-	case SERIAL_PORT_2:
-		fprintf(txt, "serial port #2\n");
-		break;
-	case SERIAL_PORT_3:
-		fprintf(txt, "serial port #3\n");
-		break;
-	case SERIAL_PORT_4:
-		fprintf(txt, "serial port #4\n");
-		break;
-	case VIRTUAL_PORT_1:
-		fprintf(txt, "virtual port #1\n");
-		break;
-	case VIRTUAL_PORT_2:
-		fprintf(txt, "virtual port #2\n");
-		break;
-	case USB_PORT_1:
-		fprintf(txt, "USB port #1\n");
-		break;
-	case USB_PORT_2:
-		fprintf(txt, "USB port #2\n");
-		break;
-	case USB_PORT_3:
-		fprintf(txt, "USB port #3\n");
-		break;
-	case USB_PORT_4:
-		fprintf(txt, "USB port #4\n");
-		break;
-	default:
-		fprintf(txt, "null\n");
-		break;
-	}
-	fprintf(txt, "\n");
-
-	fprintf(txt, "# Method to use for I/O accesses\n");
-	fprintf(txt, "method=");
-	if (link_cable.method & IOM_AUTO)
-		fprintf(txt, "automatic\n");
-
-	else if (link_cable.method & IOM_ASM)
-		fprintf(txt, "asm\n");
-
-	else if (link_cable.method & IOM_API)
-		fprintf(txt, "api\n");
-
-	else if (link_cable.method & IOM_DRV)
-		fprintf(txt, "driver\n");
-	else if (link_cable.method & IOM_NULL)
-		fprintf(txt, "null\n");
-	else
-		fprintf(txt, "automatic\n");
-	fprintf(txt, "\n");
-
-	fprintf(txt, "# Parallel/serial/virtual port address (0=automatic)\n");
-	fprintf(txt, "adr_port=0x%03X\n", link_cable.io_addr);
-	fprintf(txt, "\n");
-
-	fprintf(txt, "# Serial device or character device (empty=automatic)\n");
-	fprintf(txt, "serial_device=%s\n", link_cable.device);
+	fprintf(txt, "cable_port=%s\n", ticables_port_to_string(linkp.cable_port));
 	fprintf(txt, "\n");
 
 	fprintf(txt, "# Timeout value in 0.1 seconds\n");
-	fprintf(txt, "timeout=%i\n", link_cable.timeout);
+	fprintf(txt, "cable_timeout=%i\n", linkp.cable_timeout);
 	fprintf(txt, "\n");
 
 	fprintf(txt, "# Delay value\n");
-	fprintf(txt, "delay=%i\n", link_cable.delay);
+	fprintf(txt, "cable_delay=%i\n", linkp.cable_delay);
+	fprintf(txt, "\n");
+
+	fprintf(txt, "# Quick-send feature\n");
+	fprintf(txt, "qs_file=%s\n", options.qs_file);
+	fprintf(txt, "qs_enabled=%i\n", options.qs_enabled);
 	fprintf(txt, "\n");
 
   /* Specific part to TiEmu */
@@ -833,7 +662,7 @@ void rcfile_write(void)
   fprintf(txt, "#\n");
   fprintf(txt, "\n");
 
-  fprintf(txt, "# Screenshot: image format (xpm, pcx, jpg, bmp)\n");
+  fprintf(txt, "# Screenshot: image format (jpg, bmp, ico, eps, pdf, bmp)\n");
   fprintf(txt, "img_format=");
   switch(options2.format)
     {
@@ -842,6 +671,7 @@ void rcfile_write(void)
     case IMG_ICO: fprintf(txt, "ico\n"); break;
     case IMG_EPS: fprintf(txt, "eps\n"); break;
     case IMG_PDF: fprintf(txt, "pdf\n"); break;
+	case IMG_BMP: fprintf(txt, "bmp\n"); break;
     }
   fprintf(txt, "\n");
 
@@ -853,6 +683,10 @@ void rcfile_write(void)
   fprintf(txt, "img_size=%s\n", (options2.size == IMG_LCD) ? "lcd" : "skin");
   fprintf(txt, "\n");
 
+  fprintf(txt, "# Screenshot folder\n");
+  fprintf(txt, "screen_folder=%s\n", options2.folder);
+  fprintf(txt, "\n");
+
   fprintf(txt, "# Screenshot base file\n");
   fprintf(txt, "screen_file=%s\n", options2.file);
   fprintf(txt, "\n");
@@ -861,6 +695,24 @@ void rcfile_write(void)
   fprintf(txt, "screen_counter=%i\n", options2.counter);
   fprintf(txt, "\n");
 
+  fprintf(txt, "# Screenshot shots\n");
+  fprintf(txt, "screen_shots=%i\n", options2.shots);
+  fprintf(txt, "\n");
+
+  fprintf(txt, "# Screenshot skips\n");
+  fprintf(txt, "screen_skips=%i\n", options2.skips);
+  fprintf(txt, "\n");
+
+  fprintf(txt, "# Screenshot copied to clipboard (no, yes)\n");
+  fprintf(txt, "clipboard=");
+  switch(options2.clipboard)
+  {
+  case 0: fprintf(txt, "no\n"); break;
+  case 1: fprintf(txt, "yes\n"); break;
+  default: fprintf(txt, "no\n"); break;
+  }
+  fprintf(txt, "\n");
+
   fprintf(txt, "#\n");
   fprintf(txt, "# MISC SECTION\n");
   fprintf(txt, "#\n");
@@ -881,13 +733,13 @@ void rcfile_write(void)
   fprintf(txt, "kbd_dbg=%s\n", options.kbd_dbg ? "yes" : "no");
   fprintf(txt, "\n");
 
-  fprintf(txt, "# File selector to use (old (GTK1/2) or new (GTK 2.4+) or win32 (Windows) or kde (KDE))\n");
+  fprintf(txt, "# File selector to use (native or old (GTK1/2) or new (GTK 2.4+) or win32 (Windows) or kde (KDE))\n");
   fprintf(txt, "fs_type=");
   switch(options.fs_type)
   {
   case 0: fprintf(txt, "old\n"); break;
   case 1: fprintf(txt, "new\n"); break;
-  case 2: fprintf(txt, "win32\n"); break;
+  case 2: fprintf(txt, "native\n"); break;
   case 3: fprintf(txt, "kde\n"); break;
   default: fprintf(txt, "old\n"); break;
   }
@@ -954,6 +806,39 @@ void rcfile_write(void)
 		options3.iop.minimized, options3.iop.closed);
 	fprintf(txt, "\n");
 
+	fprintf(txt, "dock_wnd=(%i;%i;%i;%i;%i;%i)", 
+		options3.dock.rect.x, options3.dock.rect.y, 
+		options3.dock.rect.w, options3.dock.rect.h,
+		options3.dock.minimized, options3.dock.closed);
+	fprintf(txt, "\n");
+
+	fprintf(txt, "wnd_transient=%s\n", options3.transient ? "yes" : "no");
+ 	fprintf(txt, "\n");
+
+	fprintf(txt, "# Font type\n");
+	fprintf(txt, "dbg_font_type=");
+	switch(options3.dbg_font_type)
+	{
+	case 0: fprintf(txt, "default\n"); break;
+	case 1: fprintf(txt, "custom\n"); break;
+	default: fprintf(txt, "default\n"); break;
+	}
+	if(options3.dbg_font_name)
+	{
+		fprintf(txt, "dbg_font_name=%s\n", options3.dbg_font_name);
+ 		fprintf(txt, "\n");
+	}
+
+	fprintf(txt, "# Use dock (yes) or multi-window (no)\n");
+	fprintf(txt, "dbg_dock=");
+	switch(options3.dbg_dock)
+	{
+	case 0: fprintf(txt, "no\n"); break;
+	case 1: fprintf(txt, "yes\n"); break;
+	default: fprintf(txt, "no\n"); break;
+	}
+	fprintf(txt, "\n");
+
 	fprintf(txt, "\n");
   fprintf(txt, "RC_END\n");
   fflush(txt);
@@ -991,22 +876,21 @@ int rcfile_default()
 	options.view = VIEW_NORMAL;
 	options.scale = 1.0;
 	options.kbd_dbg = 0;
-#ifdef __WIN32__
 	options.fs_type = 2;
-#else
-#if WITH_KDE
-	options.fs_type = 3;
-#else
-	options.fs_type = 1;
-#endif
-#endif
+
+	options.qs_file = g_strdup("");
+	options.qs_enabled = 0;
 
 	// screenshot options
+	options2.folder = g_strdup(inst_paths.screen_dir);
 	options2.file = g_strdup("screenshot");
 	options2.counter = 0;
-	options2.type = IMG_COL;	//IMG_BW;
+	options2.type = IMG_COL;
 	options2.format = IMG_PNG;
-	options2.size = IMG_SKIN;	//IMG_LCD;
+	options2.size = IMG_SKIN;
+	options2.shots = 1;
+	options2.skips = 4;
+	options2.clipboard = 0;
 
 	// debugger options
 	options3_set_default();	
@@ -1079,4 +963,18 @@ void options3_set_default(void)
 	options3.iop.rect.h = 240;
 	options3.iop.closed = !0;
 	options3.iop.minimized = 0;
+
+	options3.dock.rect.x = 0;
+	options3.dock.rect.y = 0;
+	options3.dock.rect.w = -1;	// unused yet
+	options3.dock.rect.h = -1;	// unused yet
+	options3.dock.closed = 0;
+	options3.dock.minimized = 0;
+
+	options3.transient = 0;
+	
+	options3.dbg_font_type = 0;
+	options3.dbg_font_name = g_strdup(DEFAULT_FONT_NAME);
+
+	options3.dbg_dock = 1;
 }
